"""
Physics forces submodule
"""
from __future__ import annotations
import basilisk
import glm
import typing
__all__: list[str] = ['Contact', 'Edges', 'FeaturePair', 'Force', 'Joint', 'Manifold', 'Motor', 'Spring']
class Contact:
    C0: glm.vec2
    feature: ...
    normal: glm.vec2
    rA: glm.vec2
    rB: glm.vec2
    stick: bool
class Edges:
    inEdge1: str
    inEdge2: str
    outEdge1: str
    outEdge2: str
class FeaturePair:
    e: ...
    @property
    def value(self) -> int:
        ...
    @value.setter
    def value(self, arg0: typing.SupportsInt) -> None:
        ...
class Force:
    def disable(self) -> None:
        ...
class Joint(Force):
    def __init__(self, solver: basilisk.Solver, bodyA: basilisk.Rigid, bodyB: basilisk.Rigid, rA: glm.vec2, rB: glm.vec2, stiffness: glm.vec3 = ..., fracture: typing.SupportsFloat = ...) -> None:
        ...
    def getC0(self) -> glm.vec3:
        ...
    def getRA(self) -> glm.vec2:
        ...
    def getRB(self) -> glm.vec2:
        ...
    def getRestAngle(self) -> float:
        ...
    def getTorqueArm(self) -> float:
        ...
    def setC0(self, arg0: glm.vec3) -> None:
        ...
    def setRA(self, arg0: glm.vec2) -> None:
        ...
    def setRB(self, arg0: glm.vec2) -> None:
        ...
    def setRestAngle(self, arg0: typing.SupportsFloat) -> None:
        ...
    def setTorqueArm(self, arg0: typing.SupportsFloat) -> None:
        ...
class Manifold(Force):
    @staticmethod
    def collide(bodyA: basilisk.Rigid, bodyB: basilisk.Rigid, contacts: Contact) -> int:
        ...
    def __init__(self, solver: basilisk.Solver, bodyA: basilisk.Rigid, bodyB: basilisk.Rigid) -> None:
        ...
    def getContact(self, arg0: typing.SupportsInt) -> Contact:
        ...
    def getContactRef(self, arg0: typing.SupportsInt) -> Contact:
        ...
    def getFriction(self) -> float:
        ...
    def getNumContacts(self) -> int:
        ...
class Motor(Force):
    def __init__(self, solver: basilisk.Solver, bodyA: basilisk.Rigid, bodyB: basilisk.Rigid, speed: typing.SupportsFloat, maxTorque: typing.SupportsFloat) -> None:
        ...
    def getSpeed(self) -> float:
        ...
    def setSpeed(self, arg0: typing.SupportsFloat) -> None:
        ...
class Spring(Force):
    def __init__(self, solver: basilisk.Solver, bodyA: basilisk.Rigid, bodyB: basilisk.Rigid, rA: glm.vec2, rB: glm.vec2, stiffness: typing.SupportsFloat, rest: typing.SupportsFloat = -1.0) -> None:
        ...
    def getRA(self) -> glm.vec2:
        ...
    def getRB(self) -> glm.vec2:
        ...
    def getRest(self) -> float:
        ...
    def setRA(self, arg0: glm.vec2) -> None:
        ...
    def setRB(self, arg0: glm.vec2) -> None:
        ...
    def setRest(self, arg0: typing.SupportsFloat) -> None:
        ...



import basilisk as bsk
import random

engine = bsk.Engine(800, 800, "Basilisk")
scene = bsk.Scene(engine)

img = bsk.Image("textures/container.jpg")
image = bsk.Image([1.0, 0.0, 0.0, 1.0], 1, 1)
mtl = bsk.Material((1, 1, 1), image, image)
mesh = bsk.Mesh("models/cube.obj", False, False)


# Make node
node = bsk.Node(scene, mesh, mtl, (0, 0, 0), (0, 0, 0, 1), (1, 1, 1))
scene.add(node)

# Add ambient light
light1 = bsk.AmbientLight((1, 1, 1), 0.2)
scene.add(light1)

# Add directional light
light2 = bsk.DirectionalLight((1, 1, 1), 1.0, (0, -1, 0.2))
scene.add(light2)

# Add point light
light3 = bsk.PointLight((1, 1, 1), 1.0, (0, 0, 0), 8.0)
scene.add(light3)

# Add skybox
cubemap = bsk.Cubemap(["textures/skybox/right.jpg", "textures/skybox/left.jpg", "textures/skybox/top.jpg", "textures/skybox/bottom.jpg", "textures/skybox/front.jpg", "textures/skybox/back.jpg"])
skybox = bsk.Skybox(cubemap)
scene.set_skybox(skybox)

while engine.is_running():
    engine.update()
    scene.update()
    scene.render()
    engine.render()


"""
Node hierarchy matrix stack test with face tracking
"""

import basilisk as bsk
import glm
from math import sin, cos
import cv2
from collections import deque

# Set up face detection
face_cascade = cv2.CascadeClassifier(cv2.data.haarcascades + 'haarcascade_frontalface_default.xml')
cap = cv2.VideoCapture(0)

if not cap.isOpened():
    print("Warning: Could not open webcam", file=__import__('sys').stderr)

engine = bsk.Engine()
scene = bsk.Scene(engine, False, True, False)

camera = bsk.StaticCamera(engine)
scene.set_camera(camera)

camera.set_position((-20, 4, 4))

model = bsk.Mesh("models/john.obj")

node1 = bsk.Node(scene)
node2 = bsk.Node(node1, mesh=model, position=(0, 2, 3))
node3 = bsk.Node(node2, position=(0, 2, 3))

# Store past face positions and sizes for averaging
FACE_HISTORY_SIZE = 15  # Number of frames to average over
face_history = deque(maxlen=FACE_HISTORY_SIZE)  # Stores (x, y, size)

t = 0

try:
    while engine.is_running():
        engine.update()
        scene.update()

        # Capture and detect face
        if cap.isOpened():
            ret, frame = cap.read()
            if ret:
                gray = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
                faces = face_cascade.detectMultiScale(
                    gray,
                    scaleFactor=1.1,
                    minNeighbors=5,
                    minSize=(30, 30)
                )
                
                if len(faces) > 0:
                    # Use the first detected face
                    x, y, w, h = faces[0]
                    # Use center of face for better tracking
                    face_x = x + w / 2  # Center x coordinate
                    face_y = y + h / 2  # Center y coordinate
                    # Use face size (area or average of width/height) as depth proxy
                    # Larger face = closer, smaller face = farther
                    face_size = (w + h) / 2  # Average of width and height
                    # Add to history (x, y, size)
                    face_history.append((face_x, face_y, face_size))
                else:
                    # If no face detected, keep the last known position in history
                    # (don't add anything, just use existing history)
                    pass
                
                # Average the face positions and size over the past n frames
                if len(face_history) > 0:
                    avg_x = sum(pos[0] for pos in face_history) / len(face_history)
                    avg_y = sum(pos[1] for pos in face_history) / len(face_history)
                    avg_size = sum(pos[2] for pos in face_history) / len(face_history)
                    # Convert face size to depth: larger size = closer (less negative z)
                    # Adjust these scaling factors as needed
                    base_z = -20
                    depth_offset = (avg_size - 100) / 20  # Adjust based on typical face size

                    camera_z = depth_offset -20
                    camera_x = -avg_x / 40 + 8
                    camera_y = -avg_y / 40 + 10
                    # Set camera position based on averaged face position and depth
                    camera.set_position((camera_z, camera_y, camera_x))


        scene.render()
        engine.render()

finally:
    # Clean up
    if cap.isOpened():
        cap.release()
    cv2.destroyAllWindows()


"""
Node hierarchy matrix stack test
"""

import basilisk as bsk
import glm
from math import sin, cos

engine = bsk.Engine()
scene = bsk.Scene(engine)

node1 = bsk.Node(scene)
node2 = bsk.Node(node1, position=(3, 2, 0))
node3 = bsk.Node(node2, position=(3, 2, 0))

t = 0

while engine.is_running():
    engine.update()
    scene.update()

    t += engine.get_delta_time()

    # Move the base side to side
    x = sin(t) * 2
    y = 0
    z = cos(t) * 2
    node1.set_position((x, y, z))

    # Rotate the base node around the y axis
    rot = glm.rotate(glm.quat(1, 0, 0, 0), t, (0, 1, 0))
    node1.set_rotation((rot.w, rot.x, rot.y, rot.z))

    # Rotate the second node around the X axis
    rot = glm.rotate(glm.quat(1, 0, 0, 0), t, (1, 0, 0))
    node2.set_rotation((rot.w, rot.x, rot.y, rot.z))

    # Rotate the third node around the Z axis
    rot = glm.rotate(glm.quat(1, 0, 0, 0), t, (0, 0, 1))
    node3.set_rotation((rot.w, rot.x, rot.y, rot.z))

    # Scale the base node up and down
    s = cos(t) / 2 + 1
    node1.set_scale((s, s, s))

    scene.render()
    engine.render()


"""
pybind11 example plugin
"""
from __future__ import annotations
import collections.abc
import glm
import typing
import typing_extensions
__all__: list[str] = ['Collider', 'ComputeShader', 'Contact', 'EBO', 'Edges', 'Engine', 'FBO', 'FeaturePair', 'Force', 'Frame', 'GpuBuffer', 'GpuBufferDtype', 'Image', 'Joint', 'Manifold', 'Material', 'Mesh', 'Motor', 'Node', 'Node2D', 'Rigid', 'Scene', 'Shader', 'Solver', 'Spring', 'VAO', 'VBO']


def init_gpu() -> None:
    """Initialize the GPU compute backend (call once at startup)."""
    ...


class GpuBufferDtype:
    U32: int
    F32: int
    I32: int


class GpuBuffer:
    """Single GPU buffer; dtype controls element type ('u32'|'f32'|'i32'), len is element count."""
    def __init__(self, dtype: typing.Union[str, typing.Any], len: int) -> None:
        ...
    def write(self, data: collections.abc.Sequence[typing.Union[int, float]]) -> None:
        ...
    def read(self) -> list[typing.Union[int, float]]:
        ...
    def len(self) -> int:
        ...
    def size_bytes(self) -> int:
        ...
    def element_size(self) -> int:
        ...
    def dtype(self) -> str:
        """Return dtype string: 'u32', 'f32', or 'i32'."""
        ...
    def handle(self) -> int:
        """Return opaque handle for use with ComputeShader buffer_handles."""
        ...


class ComputeShader:
    def __init__(self, wgsl: str, buffer_handles: list[int], uniform_size: int = 0) -> None:
        ...
    def set_uniform_bytes(self, data: typing.Union[bytes, typing_extensions.Buffer]) -> None:
        """Set uniform data from a bytes-like or buffer object."""
        ...
    def dispatch(self, x: int, y: int = 1, z: int = 1) -> None:
        """Dispatch the compute shader with workgroup counts (x, y, z)."""
        ...
class Collider:
    def __init__(self, solver: Solver, vertices: collections.abc.Sequence[glm.vec2]) -> None:
        ...
    def getArea(self) -> float:
        ...
    def getBaseMoment(self) -> float:
        ...
    def getBaseRadius(self) -> float:
        ...
    def getCOM(self) -> glm.vec2:
        ...
    def getGC(self) -> glm.vec2:
        ...
    def getHalfDim(self) -> glm.vec2:
        ...
    def getMass(self, arg0: glm.vec2, arg1: typing.SupportsFloat) -> float:
        ...
    def getMoment(self, arg0: glm.vec2, arg1: typing.SupportsFloat) -> float:
        ...
    def getRadius(self, arg0: glm.vec2) -> float:
        ...
    def getVertices(self) -> list[glm.vec2]:
        ...
    def setArea(self, arg0: typing.SupportsFloat) -> None:
        ...
    def setBaseMoment(self, arg0: typing.SupportsFloat) -> None:
        ...
    def setCOM(self, arg0: glm.vec2) -> None:
        ...
    def setGC(self, arg0: glm.vec2) -> None:
        ...
    def setHalfDim(self, arg0: glm.vec2) -> None:
        ...
    def setVertices(self, arg0: collections.abc.Sequence[glm.vec2]) -> None:
        ...
class Contact:
    C0: glm.vec2
    feature: ...
    normal: glm.vec2
    rA: glm.vec2
    rB: glm.vec2
    stick: bool
class EBO:
    def __init__(self, data: collections.abc.Sequence[typing.SupportsInt], drawType: typing.SupportsInt = 35044) -> None:
        ...
    def bind(self) -> None:
        ...
    def getSize(self) -> int:
        ...
    def unbind(self) -> None:
        ...
    def write(self, data: typing_extensions.CapsuleType, size: typing.SupportsInt, offset: typing.SupportsInt) -> None:
        ...
class Edges:
    inEdge1: str
    inEdge2: str
    outEdge1: str
    outEdge2: str
class Engine:
    def __init__(self, width: typing.SupportsInt, height: typing.SupportsInt, title: str, autoMouseGrab: bool = True) -> None:
        ...
    def getFrame(self) -> ...:
        ...
    def getKeyboard(self) -> ...:
        ...
    def getMouse(self) -> ...:
        ...
    def getResourceServer(self) -> ...:
        ...
    def getWindow(self) -> ...:
        ...
    def isRunning(self) -> bool:
        ...
    def render(self) -> None:
        ...
    def setResolution(self, width: typing.SupportsInt, height: typing.SupportsInt) -> None:
        ...
    def update(self) -> None:
        ...
    def useContext(self) -> None:
        ...
class FBO:
    def __init__(self, width: typing.SupportsInt, height: typing.SupportsInt, components: typing.SupportsInt) -> None:
        ...
    def bind(self) -> None:
        ...
    def clear(self, r: typing.SupportsFloat, g: typing.SupportsFloat, b: typing.SupportsFloat, a: typing.SupportsFloat) -> None:
        ...
    def getDepthID(self) -> int:
        ...
    def getHeight(self) -> int:
        ...
    def getID(self) -> int:
        ...
    def getTextureID(self) -> int:
        ...
    def getWidth(self) -> int:
        ...
    def unbind(self) -> None:
        ...
class FeaturePair:
    e: ...
    @property
    def value(self) -> int:
        ...
    @value.setter
    def value(self, arg0: typing.SupportsInt) -> None:
        ...
class Force:
    def disable(self) -> None:
        ...
    def getBodyA(self) -> Rigid:
        ...
    def getBodyB(self) -> Rigid:
        ...
    def getC(self, arg0: typing.SupportsInt) -> float:
        ...
    def getFmax(self, arg0: typing.SupportsInt) -> float:
        ...
    def getFmin(self, arg0: typing.SupportsInt) -> float:
        ...
    def getFracture(self, arg0: typing.SupportsInt) -> float:
        ...
    def getH(self, arg0: typing.SupportsInt, arg1: Rigid) -> ...:
        ...
    def getJ(self, arg0: typing.SupportsInt, arg1: Rigid) -> glm.vec3:
        ...
    def getLambda(self, arg0: typing.SupportsInt) -> float:
        ...
    def getNext(self) -> Force:
        ...
    def getNextA(self) -> Force:
        ...
    def getNextB(self) -> Force:
        ...
    def getPenalty(self, arg0: typing.SupportsInt) -> float:
        ...
    def getPrev(self) -> Force:
        ...
    def getPrevA(self) -> Force:
        ...
    def getPrevB(self) -> Force:
        ...
    def getSolver(self) -> Solver:
        ...
    def getStiffness(self, arg0: typing.SupportsInt) -> float:
        ...
class Frame:
    def __init__(self, engine: Engine, width: typing.SupportsInt, height: typing.SupportsInt) -> None:
        ...
    def clear(self, r: typing.SupportsFloat, g: typing.SupportsFloat, b: typing.SupportsFloat, a: typing.SupportsFloat) -> None:
        ...
    def getAspectRatio(self) -> float:
        ...
    def getEBO(self) -> EBO:
        ...
    def getFBO(self) -> FBO:
        ...
    def getHeight(self) -> int:
        ...
    def getRenderHeight(self) -> int:
        ...
    def getRenderWidth(self) -> int:
        ...
    def getShader(self) -> Shader:
        ...
    def getVAO(self) -> VAO:
        ...
    def getVBO(self) -> VBO:
        ...
    def getWidth(self) -> int:
        ...
    @typing.overload
    def render(self) -> None:
        ...
    @typing.overload
    def render(self, x: typing.SupportsInt, y: typing.SupportsInt, width: typing.SupportsInt, height: typing.SupportsInt) -> None:
        ...
    def use(self) -> None:
        ...
class Image:
    def __init__(self, file: str) -> None:
        ...
    def getData(self) -> int:
        ...
    def getHeight(self) -> int:
        ...
    def getWidth(self) -> int:
        ...
class Joint(Force):
    def __init__(self, solver: Solver, bodyA: Rigid, bodyB: Rigid, rA: glm.vec2, rB: glm.vec2, stiffness: glm.vec3 = ..., fracture: typing.SupportsFloat = ...) -> None:
        ...
    def getC0(self) -> glm.vec3:
        ...
    def getRA(self) -> glm.vec2:
        ...
    def getRB(self) -> glm.vec2:
        ...
    def getRestAngle(self) -> float:
        ...
    def getTorqueArm(self) -> float:
        ...
    def setC0(self, arg0: glm.vec3) -> None:
        ...
    def setRA(self, arg0: glm.vec2) -> None:
        ...
    def setRB(self, arg0: glm.vec2) -> None:
        ...
    def setRestAngle(self, arg0: typing.SupportsFloat) -> None:
        ...
    def setTorqueArm(self, arg0: typing.SupportsFloat) -> None:
        ...
class Manifold(Force):
    @staticmethod
    def collide(bodyA: Rigid, bodyB: Rigid, contacts: Contact) -> int:
        ...
    def __init__(self, solver: Solver, bodyA: Rigid, bodyB: Rigid) -> None:
        ...
    def getContact(self, arg0: typing.SupportsInt) -> Contact:
        ...
    def getContactRef(self, arg0: typing.SupportsInt) -> Contact:
        ...
    def getFriction(self) -> float:
        ...
    def getNumContacts(self) -> int:
        ...
class Material:
    def __init__(self, color: glm.vec3 = ..., albedo: Image = None, normal: Image = None) -> None:
        ...
    def getAlbedo(self) -> Image:
        ...
    def getColor(self) -> glm.vec3:
        ...
    def getNormal(self) -> Image:
        ...
class Mesh:
    def __init__(self, modelPath: str, generateUV: bool = False, generateNormals: bool = False) -> None:
        ...
    def get_indices(self) -> list[int]:
        ...
    def get_vertices(self) -> list[float]:
        ...
class Motor(Force):
    def __init__(self, solver: Solver, bodyA: Rigid, bodyB: Rigid, speed: typing.SupportsFloat, maxTorque: typing.SupportsFloat) -> None:
        ...
    def getSpeed(self) -> float:
        ...
    def setSpeed(self, arg0: typing.SupportsFloat) -> None:
        ...
class Node:
    @typing.overload
    def __init__(self, scene: Scene, mesh: ... = None, material: ... = None, position: glm.vec3 = ..., rotation: glm.quat = ..., scale: glm.vec3 = ...) -> None:
        ...
    @typing.overload
    def __init__(self, parent: Node, mesh: ... = None, material: ... = None, position: glm.vec3 = ..., rotation: glm.quat = ..., scale: glm.vec3 = ...) -> None:
        ...
    @typing.overload
    def __init__(self, scene: Scene, parent: Node) -> None:
        ...
    def setPosition(self, position: glm.vec3) -> None:
        ...
    def setRotation(self, rotation: glm.quat) -> None:
        ...
    def setScale(self, scale: glm.vec3) -> None:
        ...
class Node2D:
    @typing.overload
    def __init__(self, scene: ..., mesh: Mesh, material: Material, position: glm.vec2, rotation: typing.SupportsFloat, scale: glm.vec2, velocity: glm.vec3, collider: ..., density: typing.SupportsFloat, friction: typing.SupportsFloat) -> None:
        ...
    @typing.overload
    def __init__(self, parent: Node2D, mesh: Mesh, material: Material, position: glm.vec2, rotation: typing.SupportsFloat, scale: glm.vec2, velocity: glm.vec3, collider: ..., density: typing.SupportsFloat, friction: typing.SupportsFloat) -> None:
        ...
    @typing.overload
    def __init__(self, arg0: ..., arg1: Node2D) -> None:
        ...
    @typing.overload
    def setPosition(self, arg0: glm.vec2) -> None:
        ...
    @typing.overload
    def setPosition(self, arg0: glm.vec3) -> None:
        ...
    def setRotation(self, arg0: typing.SupportsFloat) -> None:
        ...
    def setScale(self, arg0: glm.vec2) -> None:
        ...
    def setVelocity(self, arg0: glm.vec3) -> None:
        ...
class Rigid:
    def __init__(self, solver: Solver, node: Node2D, collider: Collider, position: glm.vec3, size: glm.vec2, density: typing.SupportsFloat, friction: typing.SupportsFloat, velocity: glm.vec3) -> None:
        ...
    def constrainedTo(self, arg0: Rigid) -> bool:
        ...
    def getAABB(self) -> tuple[glm.vec2, glm.vec2]:
        ...
    def getCollider(self) -> Collider:
        ...
    def getColor(self) -> int:
        ...
    def getDegree(self) -> int:
        ...
    def getDensity(self) -> float:
        ...
    def getForces(self) -> ...:
        ...
    def getFriction(self) -> float:
        ...
    def getIndex(self) -> int:
        ...
    def getInertial(self) -> glm.vec3:
        ...
    def getInitial(self) -> glm.vec3:
        ...
    def getMass(self) -> float:
        ...
    def getMoment(self) -> float:
        ...
    def getNext(self) -> Rigid:
        ...
    def getNextUnusedColor(self) -> int:
        ...
    def getNode(self) -> Node2D:
        ...
    def getPosition(self) -> glm.vec3:
        ...
    def getPrev(self) -> Rigid:
        ...
    def getPrevVelocity(self) -> glm.vec3:
        ...
    def getRadius(self) -> float:
        ...
    def getSatur(self) -> int:
        ...
    def getSize(self) -> glm.vec2:
        ...
    def getSolver(self) -> Solver:
        ...
    def getVel(self) -> glm.vec3:
        ...
    def getVelocity(self) -> glm.vec3:
        ...
    def incrSatur(self) -> None:
        ...
    def insert(self, arg0: ...) -> None:
        ...
    def isColorUsed(self, arg0: typing.SupportsInt) -> bool:
        ...
    def isColored(self) -> bool:
        ...
    def remove(self, arg0: ...) -> None:
        ...
    def reserveColors(self, arg0: typing.SupportsInt) -> None:
        ...
    def resetColoring(self) -> None:
        ...
    def setCollider(self, arg0: Collider) -> None:
        ...
    def setFriction(self, arg0: typing.SupportsFloat) -> None:
        ...
    def setInertial(self, arg0: glm.vec3) -> None:
        ...
    def setInitial(self, arg0: glm.vec3) -> None:
        ...
    def setMass(self, arg0: typing.SupportsFloat) -> None:
        ...
    def setMoment(self, arg0: typing.SupportsFloat) -> None:
        ...
    def setNode(self, arg0: Node2D) -> None:
        ...
    def setPosition(self, arg0: glm.vec3) -> None:
        ...
    def setPrevVelocity(self, arg0: glm.vec3) -> None:
        ...
    def setRadius(self, arg0: typing.SupportsFloat) -> None:
        ...
    def setScale(self, arg0: glm.vec2) -> None:
        ...
    def setVelocity(self, arg0: glm.vec3) -> None:
        ...
    def useColor(self, arg0: typing.SupportsInt) -> None:
        ...
    def verifyColoring(self) -> bool:
        ...
class Scene:
    def __init__(self, engine: Engine) -> None:
        ...
    def getCamera(self) -> ...:
        ...
    def getShader(self) -> ...:
        ...
    def render(self) -> None:
        ...
    def setCamera(self, camera: ...) -> None:
        ...
    def update(self) -> None:
        ...
class Shader:
    @staticmethod
    @typing.overload
    def setUniform(*args, **kwargs) -> None:
        ...
    def __init__(self, vertexPath: str, fragmentPath: str) -> None:
        ...
    @typing.overload
    def bind(self, name: str, texture: ..., slot: typing.SupportsInt) -> None:
        ...
    @typing.overload
    def bind(self, name: str, textureArray: ..., slot: typing.SupportsInt) -> None:
        ...
    @typing.overload
    def bind(self, name: str, tbo: ..., slot: typing.SupportsInt) -> None:
        ...
    @typing.overload
    def bind(self, name: str, fbo: ..., slot: typing.SupportsInt) -> None:
        ...
    def getAttributes(self) -> ...:
        ...
    def getStride(self) -> int:
        ...
    def getUniformLocation(self, name: str) -> int:
        ...
    @typing.overload
    def setUniform(self, name: str, value: typing.SupportsFloat) -> None:
        ...
    @typing.overload
    def setUniform(self, name: str, value: typing.SupportsFloat) -> None:
        ...
    @typing.overload
    def setUniform(self, name: str, value: typing.SupportsInt) -> None:
        ...
    @typing.overload
    def setUniform(self, name: str, value: glm.vec3) -> None:
        ...
    def use(self) -> None:
        ...
class Solver:
    @staticmethod
    def setGravity(*args, **kwargs) -> None:
        ...
    def __init__(self) -> None:
        ...
    def getAlpha(self) -> float:
        ...
    def getBeta(self) -> float:
        ...
    def getDt(self) -> float:
        ...
    def getGamma(self) -> float:
        ...
    def getGravity(self) -> ...:
        ...
    def getIterations(self) -> int:
        ...
    def getNumForces(self) -> int:
        ...
    def getNumRigids(self) -> int:
        ...
    def getPostStabilize(self) -> bool:
        ...
    @typing.overload
    def insert(self, arg0: ...) -> None:
        ...
    @typing.overload
    def insert(self, arg0: ...) -> None:
        ...
    @typing.overload
    def remove(self, arg0: ...) -> None:
        ...
    @typing.overload
    def remove(self, arg0: ...) -> None:
        ...
    def setAlpha(self, arg0: typing.SupportsFloat) -> None:
        ...
    def setBeta(self, arg0: typing.SupportsFloat) -> None:
        ...
    def setDt(self, arg0: typing.SupportsFloat) -> None:
        ...
    def setGamma(self, arg0: typing.SupportsFloat) -> None:
        ...
    def setIterations(self, arg0: typing.SupportsInt) -> None:
        ...
    def setPostStabilize(self, arg0: bool) -> None:
        ...
class Spring(Force):
    def __init__(self, solver: Solver, bodyA: Rigid, bodyB: Rigid, rA: glm.vec2, rB: glm.vec2, stiffness: typing.SupportsFloat, rest: typing.SupportsFloat = -1.0) -> None:
        ...
    def getRA(self) -> glm.vec2:
        ...
    def getRB(self) -> glm.vec2:
        ...
    def getRest(self) -> float:
        ...
    def setRA(self, arg0: glm.vec2) -> None:
        ...
    def setRB(self, arg0: glm.vec2) -> None:
        ...
    def setRest(self, arg0: typing.SupportsFloat) -> None:
        ...
class VAO:
    def __init__(self, shader: Shader, vertices: VBO, indices: EBO) -> None:
        ...
    def bind(self) -> None:
        ...
    def bindAttribute(self, location: typing.SupportsInt, count: typing.SupportsInt, dataType: typing.SupportsInt, stride: typing.SupportsInt, offset: typing.SupportsInt, divisor: typing.SupportsInt) -> None:
        ...
    def bindAttributes(self, attribs: ..., std: ..., std: ..., std: ..., std: ..., std: ..., divisor: typing.SupportsInt) -> None:
        ...
    @typing.overload
    def bindBuffer(self, buffer: VBO, attribs: ..., std: ..., std: ..., std: ..., std: ..., std: ..., divisor: typing.SupportsInt) -> None:
        ...
    @typing.overload
    def bindBuffer(self, buffer: VBO, indices: EBO, attribs: ..., std: ..., std: ..., std: ..., std: ..., std: ..., divisor: typing.SupportsInt) -> None:
        ...
    def render(self, instanceCount: typing.SupportsInt) -> None:
        ...
class VBO:
    @typing.overload
    def __init__(self, data: typing_extensions.CapsuleType, size: typing.SupportsInt, drawType: typing.SupportsInt) -> None:
        ...
    @typing.overload
    def __init__(self, data: collections.abc.Sequence[typing.SupportsFloat], drawType: typing.SupportsInt = 35044) -> None:
        ...
    def bind(self) -> None:
        ...
    def getSize(self) -> int:
        ...
    def unbind(self) -> None:
        ...



"""
pybind11 example plugin
"""
from __future__ import annotations
import collections.abc
import glm
import typing
import typing_extensions
from . import forces
from . import key
__all__: list[str] = ['AmbientLight', 'Camera', 'Camera2D', 'Collider', 'ComputeShader', 'Cubemap', 'DirectionalLight', 'EBO', 'Engine', 'F32', 'FBO', 'Frame', 'GpuBuffer', 'GpuBufferDtype', 'I32', 'Image', 'Keyboard', 'Light', 'Material', 'Mesh', 'Mouse', 'Node', 'Node2D', 'PointLight', 'RayCastResult', 'RayCastResult2D', 'Rigid', 'Scene', 'Scene2D', 'Shader', 'Skybox', 'Solver', 'StaticCamera', 'StaticCamera2D', 'U32', 'UBO', 'VAO', 'VBO', 'Window', 'forces', 'init_gpu', 'key']
class AmbientLight(Light):
    def __init__(self, color: glm.vec3 = (1.0, 1.0, 1.0), intensity: typing.SupportsFloat = 1.0) -> None:
        ...
class Camera(StaticCamera):
    def __init__(self, engine: Engine, position: glm.vec3 = (0.0, 0.0, 0.0), pitch: typing.SupportsFloat = 0.0, yaw: typing.SupportsFloat = 0.0) -> None:
        ...
    def update(self) -> None:
        ...
    @property
    def speed(self) -> float:
        ...
    @speed.setter
    def speed(self, arg1: typing.SupportsFloat) -> None:
        ...
class Camera2D(StaticCamera2D):
    def __init__(self, engine: Engine, position: glm.vec2 = (0.0, 0.0), scale: typing.SupportsFloat = 10.0) -> None:
        ...
    def update(self) -> None:
        ...
    @property
    def speed(self) -> float:
        ...
    @speed.setter
    def speed(self, arg1: typing.SupportsFloat) -> None:
        ...
class Collider:
    def __init__(self, vertices: collections.abc.Sequence[glm.vec2]) -> None:
        ...
    def getArea(self) -> float:
        ...
    def getBaseMoment(self) -> float:
        ...
    def getBaseRadius(self) -> float:
        ...
    def getCOM(self) -> glm.vec2:
        ...
    def getGC(self) -> glm.vec2:
        ...
    def getHalfDim(self) -> glm.vec2:
        ...
    def getMass(self, arg0: glm.vec2, arg1: typing.SupportsFloat) -> float:
        ...
    def getMoment(self, arg0: glm.vec2, arg1: typing.SupportsFloat) -> float:
        ...
    def getRadius(self, arg0: glm.vec2) -> float:
        ...
    def getVertices(self) -> list[glm.vec2]:
        ...
    def setArea(self, arg0: typing.SupportsFloat) -> None:
        ...
    def setBaseMoment(self, arg0: typing.SupportsFloat) -> None:
        ...
    def setCOM(self, arg0: glm.vec2) -> None:
        ...
    def setGC(self, arg0: glm.vec2) -> None:
        ...
    def setHalfDim(self, arg0: glm.vec2) -> None:
        ...
    def setVertices(self, arg0: collections.abc.Sequence[glm.vec2]) -> None:
        ...
class ComputeShader:
    def __init__(self, wgsl: str, buffer_handles: list, uniform_size: typing.SupportsInt = 0) -> None:
        ...
    def dispatch(self, x: typing.SupportsInt, y: typing.SupportsInt = 1, z: typing.SupportsInt = 1) -> None:
        """
        Dispatch the compute shader with workgroup counts (x, y, z).
        """
    def set_uniform_bytes(self, data: typing_extensions.Buffer) -> None:
        """
        Set uniform data from a bytes-like or buffer object.
        """
class Cubemap:
    @typing.overload
    def __init__(self, faces: collections.abc.Sequence[Image]) -> None:
        ...
    @typing.overload
    def __init__(self, faces: collections.abc.Sequence[str]) -> None:
        ...
    def bind(self) -> None:
        ...
    def get_id(self) -> int:
        ...
    def unbind(self) -> None:
        ...
class DirectionalLight(Light):
    def __init__(self, color: glm.vec3 = (1.0, 1.0, 1.0), intensity: typing.SupportsFloat = 1.0, direction: glm.vec3 = (0.0, -1.0, 0.20000000298023224)) -> None:
        ...
    def get_direction(self) -> glm.vec3:
        ...
    def set_direction(self, direction: glm.vec3) -> None:
        ...
class EBO:
    def __init__(self, data: collections.abc.Sequence[typing.SupportsInt], drawType: typing.SupportsInt = 35044) -> None:
        ...
    def bind(self) -> None:
        ...
    def get_size(self) -> int:
        ...
    def unbind(self) -> None:
        ...
    def write(self, data: typing_extensions.CapsuleType, size: typing.SupportsInt, offset: typing.SupportsInt) -> None:
        ...
class Engine:
    @staticmethod
    def get_resource_server() -> ...:
        ...
    def __init__(self, width: typing.SupportsInt = 800, height: typing.SupportsInt = 800, title: str = 'Basilisk', auto_mouse_grab: bool = True, show_splash: bool = False) -> None:
        ...
    def disable_blend(self) -> None:
        ...
    def disable_cull_face(self) -> None:
        ...
    def disable_depth_test(self) -> None:
        ...
    def disable_multisample(self) -> None:
        ...
    def disable_vsync(self) -> None:
        ...
    def enable_blend(self) -> None:
        ...
    def enable_cull_face(self) -> None:
        ...
    def enable_depth_test(self) -> None:
        ...
    def enable_multisample(self) -> None:
        ...
    def enable_vsync(self) -> None:
        ...
    def get_delta_time(self) -> float:
        ...
    def get_frame(self) -> ...:
        ...
    def get_keyboard(self) -> Keyboard:
        """
        Get the keyboard. Use get_pressed, get_down, get_released with basilisk.key constants.
        """
    def get_mouse(self) -> Mouse:
        """
        Get the mouse. Access position, clicks, and button states.
        """
    def get_mouse_world_x(self, camera: ...) -> float:
        """
        Mouse X in world coordinates. Pass the scene's camera, e.g. scene.get_camera().
        """
    def get_mouse_world_y(self, camera: ...) -> float:
        """
        Mouse Y in world coordinates. Pass the scene's camera, e.g. scene.get_camera().
        """
    def get_mouse_x(self) -> float:
        """
        Mouse X position (screen coordinates, scaled).
        """
    def get_mouse_y(self) -> float:
        """
        Mouse Y position (screen coordinates, scaled).
        """
    def get_window(self) -> Window:
        """
        Get the window. Access width, height, FPS, etc.
        """
    def is_key_down(self, key_code: key.KeyCode) -> bool:
        """
        True if key is currently held down.
        """
    def is_key_pressed(self, key_code: key.KeyCode) -> bool:
        """
        True if key was pressed this frame (down now, was up last frame). Use basilisk.key.K_W, etc.
        """
    def is_key_released(self, key_code: key.KeyCode) -> bool:
        """
        True if key was released this frame.
        """
    def is_left_clicked(self) -> bool:
        """
        True if left button was clicked this frame (down now, was up last frame).
        """
    def is_left_down(self) -> bool:
        """
        True if left mouse button is held down.
        """
    def is_left_released(self) -> bool:
        """
        True if left button was released this frame.
        """
    def is_middle_clicked(self) -> bool:
        """
        True if middle button was clicked this frame.
        """
    def is_middle_down(self) -> bool:
        """
        True if middle mouse button is held down.
        """
    def is_middle_released(self) -> bool:
        """
        True if middle button was released this frame.
        """
    def is_right_clicked(self) -> bool:
        """
        True if right button was clicked this frame.
        """
    def is_right_down(self) -> bool:
        """
        True if right mouse button is held down.
        """
    def is_right_released(self) -> bool:
        """
        True if right button was released this frame.
        """
    def is_running(self) -> bool:
        ...
    def render(self) -> None:
        ...
    def set_resolution(self, width: typing.SupportsInt, height: typing.SupportsInt) -> None:
        ...
    def show_splash(self) -> None:
        ...
    def update(self) -> None:
        ...
    def use_context(self) -> None:
        ...
class FBO:
    def __init__(self, width: typing.SupportsInt, height: typing.SupportsInt, components: typing.SupportsInt) -> None:
        ...
    def bind(self) -> None:
        ...
    def clear(self, r: typing.SupportsFloat, g: typing.SupportsFloat, b: typing.SupportsFloat, a: typing.SupportsFloat) -> None:
        ...
    def get_depth_id(self) -> int:
        ...
    def get_height(self) -> int:
        ...
    def get_id(self) -> int:
        ...
    def get_texture_id(self) -> int:
        ...
    def get_width(self) -> int:
        ...
    def unbind(self) -> None:
        ...
class Frame:
    @typing.overload
    def __init__(self, engine: Engine, width: typing.SupportsInt = 800, height: typing.SupportsInt = 800) -> None:
        ...
    @typing.overload
    def __init__(self, engine: Engine, shader: Shader, width: typing.SupportsInt = 800, height: typing.SupportsInt = 800) -> None:
        ...
    def clear(self, r: typing.SupportsFloat = 0.0, g: typing.SupportsFloat = 0.0, b: typing.SupportsFloat = 0.0, a: typing.SupportsFloat = 1.0) -> None:
        ...
    def get_aspect_ratio(self) -> float:
        ...
    def get_ebo(self) -> EBO:
        ...
    def get_fbo(self) -> FBO:
        ...
    def get_height(self) -> int:
        ...
    def get_render_height(self) -> int:
        ...
    def get_render_width(self) -> int:
        ...
    def get_shader(self) -> Shader:
        ...
    def get_vao(self) -> VAO:
        ...
    def get_vbo(self) -> VBO:
        ...
    def get_width(self) -> int:
        ...
    @typing.overload
    def render(self) -> None:
        ...
    @typing.overload
    def render(self, x: typing.SupportsInt, y: typing.SupportsInt, width: typing.SupportsInt, height: typing.SupportsInt) -> None:
        ...
    def set_filter_linear(self) -> None:
        ...
    def set_filter_nearest(self) -> None:
        ...
    def use(self) -> None:
        ...
class GpuBuffer:
    def __init__(self, dtype: typing.Any, len: typing.SupportsInt) -> None:
        """
        Create a GPU buffer. dtype: 'u32'|'f32'|'i32', type (int/float), or object with .name; len: element count.
        """
    def dtype(self) -> str:
        """
        Return dtype string: 'u32', 'f32', or 'i32'.
        """
    def element_size(self) -> int:
        ...
    def handle(self) -> int:
        """
        Return opaque handle for use with ComputeShader buffer_handles.
        """
    def len(self) -> int:
        ...
    def read(self) -> list:
        """
        Read buffer into a list of elements.
        """
    def size_bytes(self) -> int:
        ...
    def write(self, data: collections.abc.Sequence) -> None:
        """
        Write a sequence of elements (length must not exceed buffer len).
        """
class GpuBufferDtype:
    """
    Members:
    
      U32
    
      F32
    
      I32
    """
    F32: typing.ClassVar[GpuBufferDtype]  # value = <GpuBufferDtype.F32: 1>
    I32: typing.ClassVar[GpuBufferDtype]  # value = <GpuBufferDtype.I32: 2>
    U32: typing.ClassVar[GpuBufferDtype]  # value = <GpuBufferDtype.U32: 0>
    __members__: typing.ClassVar[dict[str, GpuBufferDtype]]  # value = {'U32': <GpuBufferDtype.U32: 0>, 'F32': <GpuBufferDtype.F32: 1>, 'I32': <GpuBufferDtype.I32: 2>}
    def __eq__(self, other: typing.Any) -> bool:
        ...
    def __getstate__(self) -> int:
        ...
    def __hash__(self) -> int:
        ...
    def __index__(self) -> int:
        ...
    def __init__(self, value: typing.SupportsInt) -> None:
        ...
    def __int__(self) -> int:
        ...
    def __ne__(self, other: typing.Any) -> bool:
        ...
    def __repr__(self) -> str:
        ...
    def __setstate__(self, state: typing.SupportsInt) -> None:
        ...
    def __str__(self) -> str:
        ...
    @property
    def name(self) -> str:
        ...
    @property
    def value(self) -> int:
        ...
class Image:
    @typing.overload
    def __init__(self, file: str) -> None:
        ...
    @typing.overload
    def __init__(self, data: collections.abc.Sequence[typing.SupportsFloat], width: typing.SupportsInt, height: typing.SupportsInt, nChannels: typing.SupportsInt = 4) -> None:
        ...
    def get_data(self) -> int:
        ...
    def get_height(self) -> int:
        ...
    def get_width(self) -> int:
        ...
class Keyboard:
    def get_down(self, keyCode: key.KeyCode) -> bool:
        ...
    def get_pressed(self, keyCode: key.KeyCode) -> bool:
        ...
    def get_released(self, keyCode: key.KeyCode) -> bool:
        ...
    def update(self) -> None:
        ...
class Light:
    def get_color(self) -> glm.vec3:
        ...
    def get_intensity(self) -> float:
        ...
    def set_color(self, color: glm.vec3) -> None:
        ...
    def set_intensity(self, intensity: typing.SupportsFloat) -> None:
        ...
class Material:
    def __init__(self, color: typing.Any = None, albedo: Image = None, normal: Image = None, alpha: typing.SupportsFloat = 1.0, subsurface: typing.SupportsFloat = 0.0, sheen: typing.SupportsFloat = 0.0, sheen_tint: typing.SupportsFloat = 0.0, anisotropic: typing.SupportsFloat = 0.0, specular: typing.SupportsFloat = 0.75, metallicness: typing.SupportsFloat = 0.0, clearcoat: typing.SupportsFloat = 0.0, clearcoat_gloss: typing.SupportsFloat = 0.0) -> None:
        ...
    def get_albedo(self) -> Image:
        ...
    def get_alpha(self) -> float:
        ...
    def get_anisotropic(self) -> float:
        ...
    def get_clearcoat(self) -> float:
        ...
    def get_clearcoat_gloss(self) -> float:
        ...
    def get_color(self) -> glm.vec3:
        ...
    def get_metallicness(self) -> float:
        ...
    def get_normal(self) -> Image:
        ...
    def get_roughness(self) -> float:
        ...
    def get_sheen(self) -> float:
        ...
    def get_sheen_tint(self) -> float:
        ...
    def get_specular(self) -> float:
        ...
    def get_subsurface(self) -> float:
        ...
    @typing.overload
    def set_albedo(self, albedo: Image) -> None:
        ...
    @typing.overload
    def set_albedo(self, albedo: Image) -> None:
        ...
    def set_alpha(self, alpha: typing.SupportsFloat) -> None:
        ...
    def set_anisotropic(self, anisotropic: typing.SupportsFloat) -> None:
        ...
    def set_clearcoat(self, clearcoat: typing.SupportsFloat) -> None:
        ...
    def set_clearcoat_gloss(self, clearcoat_gloss: typing.SupportsFloat) -> None:
        ...
    def set_color(self, color: glm.vec3) -> None:
        ...
    def set_metallicness(self, metallicness: typing.SupportsFloat) -> None:
        ...
    @typing.overload
    def set_normal(self, normal: Image) -> None:
        ...
    @typing.overload
    def set_normal(self, normal: Image) -> None:
        ...
    def set_roughness(self, roughness: typing.SupportsFloat) -> None:
        ...
    def set_sheen(self, sheen: typing.SupportsFloat) -> None:
        ...
    def set_sheen_tint(self, sheen_tint: typing.SupportsFloat) -> None:
        ...
    def set_specular(self, specular: typing.SupportsFloat) -> None:
        ...
    def set_subsurface(self, subsurface: typing.SupportsFloat) -> None:
        ...
class Mesh:
    def __init__(self, modelPath: str, generateUV: bool = False, generateNormals: bool = False) -> None:
        ...
    def get_indices(self) -> list[int]:
        ...
    def get_vertices(self) -> list[float]:
        ...
class Mouse:
    def get_clicked(self) -> bool:
        ...
    def get_left_clicked(self) -> bool:
        ...
    def get_left_down(self) -> bool:
        ...
    def get_left_released(self) -> bool:
        ...
    def get_middle_clicked(self) -> bool:
        ...
    def get_middle_down(self) -> bool:
        ...
    def get_middle_released(self) -> bool:
        ...
    def get_relative_x(self) -> float:
        ...
    def get_relative_y(self) -> float:
        ...
    def get_right_clicked(self) -> bool:
        ...
    def get_right_down(self) -> bool:
        ...
    def get_right_released(self) -> bool:
        ...
    def get_world_x(self, camera: ...) -> float:
        """
        Mouse X in world coordinates using the given 2D camera.
        """
    def get_world_y(self, camera: ...) -> float:
        """
        Mouse Y in world coordinates using the given 2D camera.
        """
    def get_x(self) -> float:
        ...
    def get_y(self) -> float:
        ...
    def set_grab(self) -> None:
        ...
    def set_hidden(self) -> None:
        ...
    def set_visible(self) -> None:
        ...
    def update(self) -> None:
        ...
class Node:
    @typing.overload
    def __init__(self, scene: ..., mesh: typing.Any = None, material: typing.Any = None, position: glm.vec3 = (0.0, 0.0, 0.0), rotation: glm.quat = (1.0, 0.0, 0.0, 0.0), scale: glm.vec3 = (1.0, 1.0, 1.0)) -> None:
        ...
    @typing.overload
    def __init__(self, parent: Node, mesh: typing.Any = None, material: typing.Any = None, position: glm.vec3 = (0.0, 0.0, 0.0), rotation: glm.quat = (1.0, 0.0, 0.0, 0.0), scale: glm.vec3 = (1.0, 1.0, 1.0)) -> None:
        ...
    @typing.overload
    def __init__(self, scene: ...) -> None:
        ...
    @typing.overload
    def __init__(self, mesh: typing.Any = None, material: typing.Any = None, position: glm.vec3 = (0.0, 0.0, 0.0), rotation: glm.quat = (1.0, 0.0, 0.0, 0.0), scale: glm.vec3 = (1.0, 1.0, 1.0)) -> None:
        ...
    def add(self, child: Node) -> None:
        ...
    def get_children(self) -> ...:
        ...
    def get_engine(self) -> Engine:
        ...
    def get_material(self) -> ...:
        ...
    def get_mesh(self) -> ...:
        ...
    def get_parent(self) -> Node:
        ...
    def get_position(self) -> glm.vec3:
        ...
    def get_rotation(self) -> glm.quat:
        ...
    def get_scale(self) -> glm.vec3:
        ...
    def get_scene(self) -> ...:
        ...
    def get_shader(self) -> ...:
        ...
    def remove(self, child: Node) -> None:
        ...
    @typing.overload
    def set_material(self, material: ...) -> None:
        ...
    @typing.overload
    def set_material(self, material: ...) -> None:
        ...
    @typing.overload
    def set_mesh(self, mesh: ...) -> None:
        ...
    @typing.overload
    def set_mesh(self, mesh: ...) -> None:
        ...
    def set_position(self, position: glm.vec3) -> None:
        ...
    def set_rotation(self, rotation: typing.Any) -> None:
        ...
    def set_scale(self, scale: glm.vec3) -> None:
        ...
    @property
    def pos(self) -> glm.vec3:
        ...
    @property
    def position(self) -> _Vec3PropertyProxy:
        ...
    @position.setter
    def position(self, arg1: typing.Any) -> None:
        ...
    @property
    def rot(self) -> glm.quat:
        ...
    @property
    def rotation(self) -> _QuatPropertyProxy:
        ...
    @rotation.setter
    def rotation(self, arg1: typing.Any) -> None:
        ...
    @property
    def scale(self) -> _Vec3PropertyProxy:
        ...
    @scale.setter
    def scale(self, arg1: typing.Any) -> None:
        ...
    @property
    def scl(self) -> glm.vec3:
        ...
class Node2D:
    @typing.overload
    def __init__(self, scene: Scene2D, mesh: typing.Any, material: typing.Any, position: glm.vec2 = (0.0, 0.0), rotation: typing.SupportsFloat = 0.0, scale: glm.vec2 = (1.0, 1.0), velocity: glm.vec3 = (0.0, 0.0, 0.0), collider: ... = None, density: typing.SupportsFloat = 1.0, friction: typing.SupportsFloat = 0.5) -> None:
        ...
    @typing.overload
    def __init__(self, parent: Node2D, mesh: typing.Any, material: typing.Any, position: glm.vec2 = (0.0, 0.0), rotation: typing.SupportsFloat = 0.0, scale: glm.vec2 = (1.0, 1.0), velocity: glm.vec3 = (0.0, 0.0, 0.0), collider: ... = None, density: typing.SupportsFloat = 1.0, friction: typing.SupportsFloat = 0.5) -> None:
        ...
    @typing.overload
    def __init__(self, scene: Scene2D) -> None:
        ...
    @typing.overload
    def __init__(self, mesh: typing.Any = None, material: typing.Any = None, position: glm.vec2 = (0.0, 0.0), rotation: typing.SupportsFloat = 0.0, scale: glm.vec2 = (1.0, 1.0), velocity: glm.vec3 = (0.0, 0.0, 0.0), collider: ... = None, density: typing.SupportsFloat = 1.0, friction: typing.SupportsFloat = 0.5) -> None:
        ...
    def add(self, child: Node2D) -> None:
        ...
    def constrained_to(self, other: Node2D) -> ...:
        ...
    def get_children(self) -> ...:
        ...
    def get_collider(self) -> ...:
        ...
    def get_density(self) -> float:
        ...
    def get_engine(self) -> Engine:
        ...
    def get_friction(self) -> float:
        ...
    def get_layer(self) -> float:
        ...
    def get_material(self) -> Material:
        ...
    def get_mesh(self) -> Mesh:
        ...
    def get_parent(self) -> Node2D:
        ...
    def get_position(self) -> glm.vec2:
        ...
    def get_rigid(self) -> ...:
        ...
    def get_rotation(self) -> float:
        ...
    def get_scale(self) -> glm.vec2:
        ...
    def get_scene(self) -> Scene2D:
        ...
    def get_shader(self) -> ...:
        ...
    def get_velocity(self) -> glm.vec3:
        ...
    def is_touching(self, other: Node2D) -> bool:
        ...
    def just_collided(self, other: Node2D) -> bool:
        ...
    def remove(self, child: Node2D) -> None:
        ...
    def set_collider(self, collider: ...) -> None:
        ...
    def set_density(self, density: typing.SupportsFloat) -> None:
        ...
    def set_friction(self, friction: typing.SupportsFloat) -> None:
        ...
    def set_layer(self, layer: typing.SupportsFloat) -> None:
        ...
    @typing.overload
    def set_material(self, material: Material) -> None:
        ...
    @typing.overload
    def set_material(self, material: Material) -> None:
        ...
    @typing.overload
    def set_mesh(self, mesh: Mesh) -> None:
        ...
    @typing.overload
    def set_mesh(self, mesh: Mesh) -> None:
        ...
    @typing.overload
    def set_position(self, position: glm.vec2) -> None:
        ...
    @typing.overload
    def set_position(self, position: glm.vec3) -> None:
        ...
    def set_rotation(self, rotation: typing.SupportsFloat) -> None:
        ...
    def set_scale(self, scale: glm.vec2) -> None:
        ...
    def set_velocity(self, velocity: glm.vec3) -> None:
        ...
    @property
    def layer(self) -> float:
        ...
    @layer.setter
    def layer(self, arg1: typing.SupportsFloat) -> None:
        ...
    @property
    def pos(self) -> glm.vec2:
        ...
    @property
    def position(self) -> _Vec2PropertyProxy:
        ...
    @position.setter
    def position(self, arg1: typing.Any) -> None:
        ...
    @property
    def rotation(self) -> float:
        ...
    @rotation.setter
    def rotation(self, arg1: typing.SupportsFloat) -> None:
        ...
    @property
    def scale(self) -> _Vec2PropertyProxy:
        ...
    @scale.setter
    def scale(self, arg1: typing.Any) -> None:
        ...
    @property
    def scl(self) -> glm.vec2:
        ...
    @property
    def vel(self) -> glm.vec3:
        ...
    @property
    def velocity(self) -> _Vec3PropertyProxy:
        ...
    @velocity.setter
    def velocity(self, arg1: typing.Any) -> None:
        ...
class PointLight(Light):
    def __init__(self, color: glm.vec3 = (1.0, 1.0, 1.0), intensity: typing.SupportsFloat = 1.0, position: glm.vec3 = (0.0, 0.0, 0.0), range: typing.SupportsFloat = 15.0) -> None:
        ...
    def get_position(self) -> glm.vec3:
        ...
    def get_range(self) -> float:
        ...
    def set_position(self, position: glm.vec3) -> None:
        ...
    def set_range(self, range: typing.SupportsFloat) -> None:
        ...
class RayCastResult:
    @property
    def distance(self) -> float:
        ...
    @property
    def intersection(self) -> glm.vec3:
        ...
    @property
    def node(self) -> Node:
        """
        Hit node (shared_ptr) or None if no hit
        """
    @property
    def normal(self) -> glm.vec3:
        ...
class RayCastResult2D:
    @property
    def distance(self) -> float:
        ...
    @property
    def intersection(self) -> glm.vec2:
        ...
    @property
    def node(self) -> ...:
        """
        Hit node (shared_ptr) or None if no hit
        """
    @property
    def normal(self) -> glm.vec2:
        ...
class Rigid:
    def __init__(self, solver: Solver, node: Node2D, collider: Collider, position: glm.vec3, size: glm.vec2, density: typing.SupportsFloat, friction: typing.SupportsFloat, velocity: glm.vec3) -> None:
        ...
    def constrainedTo(self, arg0: Rigid) -> bool:
        ...
    def getAABB(self) -> tuple[glm.vec2, glm.vec2]:
        ...
    def getCollider(self) -> Collider:
        ...
    def getColor(self) -> int:
        ...
    def getDegree(self) -> int:
        ...
    def getDensity(self) -> float:
        ...
    def getForces(self) -> ...:
        ...
    def getFriction(self) -> float:
        ...
    def getIndex(self) -> int:
        ...
    def getInertial(self) -> glm.vec3:
        ...
    def getInitial(self) -> glm.vec3:
        ...
    def getMass(self) -> float:
        ...
    def getMoment(self) -> float:
        ...
    def getNext(self) -> Rigid:
        ...
    def getNextUnusedColor(self) -> int:
        ...
    def getNode(self) -> Node2D:
        ...
    def getPosition(self) -> glm.vec3:
        ...
    def getPrev(self) -> Rigid:
        ...
    def getPrevVelocity(self) -> glm.vec3:
        ...
    def getRadius(self) -> float:
        ...
    def getSatur(self) -> int:
        ...
    def getSize(self) -> glm.vec2:
        ...
    def getSolver(self) -> Solver:
        ...
    def getVel(self) -> glm.vec3:
        ...
    def getVelocity(self) -> glm.vec3:
        ...
    def incrSatur(self) -> None:
        ...
    def insert(self, arg0: ...) -> None:
        ...
    def isColorUsed(self, arg0: typing.SupportsInt) -> bool:
        ...
    def isColored(self) -> bool:
        ...
    def remove(self, arg0: ...) -> None:
        ...
    def reserveColors(self, arg0: typing.SupportsInt) -> None:
        ...
    def resetColoring(self) -> None:
        ...
    def setCollider(self, arg0: Collider) -> None:
        ...
    def setFriction(self, arg0: typing.SupportsFloat) -> None:
        ...
    def setInertial(self, arg0: glm.vec3) -> None:
        ...
    def setInitial(self, arg0: glm.vec3) -> None:
        ...
    def setMass(self, arg0: typing.SupportsFloat) -> None:
        ...
    def setMoment(self, arg0: typing.SupportsFloat) -> None:
        ...
    def setNode(self, arg0: Node2D) -> None:
        ...
    def setPosition(self, arg0: glm.vec3) -> None:
        ...
    def setPrevVelocity(self, arg0: glm.vec3) -> None:
        ...
    def setRadius(self, arg0: typing.SupportsFloat) -> None:
        ...
    def setScale(self, arg0: glm.vec2) -> None:
        ...
    def setVelocity(self, arg0: glm.vec3) -> None:
        ...
    def useColor(self, arg0: typing.SupportsInt) -> None:
        ...
    def verifyColoring(self) -> bool:
        ...
class Scene:
    @typing.overload
    def __init__(self, engine: Engine, addSkybox: bool = True, addLight: bool = True, addCube: bool = False) -> None:
        ...
    @typing.overload
    def __init__(self, engine: Engine, shader: ..., addSkybox: bool = True, addLight: bool = True, addCube: bool = False) -> None:
        ...
    @typing.overload
    def add(self, light: ...) -> None:
        ...
    @typing.overload
    def add(self, node: Node) -> None:
        ...
    def get_camera(self) -> StaticCamera:
        ...
    def get_shader(self) -> ...:
        ...
    def pick(self, mouse_position: glm.vec2) -> RayCastResultPy:
        """
        Cast a ray from the mouse position; returns RayCastResult with shared_ptr node. Raises if hit node lacks shared_ptr.
        """
    def raycast(self, origin: glm.vec3, direction: glm.vec3) -> RayCastResultPy:
        """
        Cast a ray in world space; returns RayCastResult with shared_ptr node. Raises if hit node lacks shared_ptr.
        """
    def remove(self, node: Node) -> None:
        ...
    def render(self) -> None:
        ...
    @typing.overload
    def set_camera(self, camera: StaticCamera) -> None:
        ...
    @typing.overload
    def set_camera(self, camera: StaticCamera) -> None:
        ...
    def set_skybox(self, skybox: ...) -> None:
        ...
    def update(self) -> None:
        ...
class Scene2D:
    camera: StaticCamera2D
    @typing.overload
    def __init__(self, engine: Engine) -> None:
        ...
    @typing.overload
    def __init__(self, engine: Engine, shader: ...) -> None:
        ...
    def add(self, node: ...) -> None:
        ...
    def get_camera(self) -> StaticCamera2D:
        ...
    def get_engine(self) -> Engine:
        ...
    def get_root(self) -> ...:
        ...
    def get_shader(self) -> ...:
        ...
    def get_solver(self) -> ...:
        ...
    def pick(self, position: glm.vec2) -> ...:
        """
        Return the topmost node at the given world position (by layer). Returns None if no hit. Raises if hit node lacks shared_ptr.
        """
    def raycast(self, origin: glm.vec2, direction: glm.vec2) -> RayCastResult2D:
        """
        Cast a ray in world space; returns RayCastResult2D with shared_ptr node. Raises if hit node lacks shared_ptr.
        """
    def remove(self, node: ...) -> None:
        ...
    def render(self) -> None:
        ...
    @typing.overload
    def set_camera(self, camera: StaticCamera2D) -> None:
        ...
    @typing.overload
    def set_camera(self, camera: StaticCamera2D) -> None:
        ...
    def update(self) -> None:
        ...
class Shader:
    @staticmethod
    @typing.overload
    def set_uniform(*args, **kwargs) -> None:
        ...
    def __init__(self, vertexPath: str, fragmentPath: str) -> None:
        ...
    @typing.overload
    def bind(self, name: str, texture: ..., slot: typing.SupportsInt) -> None:
        ...
    @typing.overload
    def bind(self, name: str, textureArray: ..., slot: typing.SupportsInt) -> None:
        ...
    @typing.overload
    def bind(self, name: str, tbo: ..., slot: typing.SupportsInt) -> None:
        ...
    @typing.overload
    def bind(self, name: str, fbo: ..., slot: typing.SupportsInt) -> None:
        ...
    @typing.overload
    def bind(self, name: str, cubemap: ..., slot: typing.SupportsInt) -> None:
        ...
    def get_attributes(self) -> ...:
        ...
    def get_stride(self) -> int:
        ...
    def get_uniform_location(self, name: str) -> int:
        ...
    @typing.overload
    def set_uniform(self, name: str, value: typing.SupportsFloat) -> None:
        ...
    @typing.overload
    def set_uniform(self, name: str, value: typing.SupportsFloat) -> None:
        ...
    @typing.overload
    def set_uniform(self, name: str, value: typing.SupportsInt) -> None:
        ...
    @typing.overload
    def set_uniform(self, name: str, value: glm.vec3) -> None:
        ...
    @typing.overload
    def set_uniform(self, name: str, value: glm.vec2) -> None:
        ...
    def use(self) -> None:
        ...
class Skybox:
    @typing.overload
    def __init__(self, cubemap: Cubemap, ownsCubemap: bool = False) -> None:
        ...
    @typing.overload
    def __init__(self, faces: collections.abc.Sequence[Image]) -> None:
        ...
    @typing.overload
    def __init__(self, faces: collections.abc.Sequence[str]) -> None:
        ...
    def get_cubemap(self) -> Cubemap:
        ...
    def get_shader(self) -> Shader:
        ...
    def get_vao(self) -> VAO:
        ...
    def get_vbo(self) -> VBO:
        ...
    def render(self, camera: StaticCamera) -> None:
        ...
class Solver:
    @staticmethod
    def setGravity(*args, **kwargs) -> None:
        ...
    def __init__(self) -> None:
        ...
    def getAlpha(self) -> float:
        ...
    def getBeta(self) -> float:
        ...
    def getDt(self) -> float:
        ...
    def getGamma(self) -> float:
        ...
    def getGravity(self) -> ...:
        ...
    def getIterations(self) -> int:
        ...
    def getNumForces(self) -> int:
        ...
    def getNumRigids(self) -> int:
        ...
    def getPostStabilize(self) -> bool:
        ...
    @typing.overload
    def insert(self, arg0: ...) -> None:
        ...
    @typing.overload
    def insert(self, arg0: ...) -> None:
        ...
    @typing.overload
    def remove(self, arg0: ...) -> None:
        ...
    @typing.overload
    def remove(self, arg0: ...) -> None:
        ...
    def setAlpha(self, arg0: typing.SupportsFloat) -> None:
        ...
    def setBeta(self, arg0: typing.SupportsFloat) -> None:
        ...
    def setDt(self, arg0: typing.SupportsFloat) -> None:
        ...
    def setGamma(self, arg0: typing.SupportsFloat) -> None:
        ...
    def setIterations(self, arg0: typing.SupportsInt) -> None:
        ...
    def setPostStabilize(self, arg0: bool) -> None:
        ...
class StaticCamera:
    def __init__(self, engine: Engine, position: glm.vec3 = (0.0, 0.0, 0.0), pitch: typing.SupportsFloat = 0.0, yaw: typing.SupportsFloat = 0.0) -> None:
        ...
    def get_aspect(self) -> float:
        ...
    def get_far(self) -> float:
        ...
    def get_fov(self) -> float:
        ...
    def get_near(self) -> float:
        ...
    def get_pitch(self) -> float:
        ...
    def get_position(self) -> glm.vec3:
        ...
    def get_x(self) -> float:
        ...
    def get_y(self) -> float:
        ...
    def get_yaw(self) -> float:
        ...
    def get_z(self) -> float:
        ...
    def look_at(self, target: glm.vec3) -> None:
        ...
    def set_aspect(self, aspect: typing.SupportsFloat) -> None:
        ...
    def set_far(self, far: typing.SupportsFloat) -> None:
        ...
    def set_fov(self, fov: typing.SupportsFloat) -> None:
        ...
    def set_near(self, near: typing.SupportsFloat) -> None:
        ...
    def set_pitch(self, pitch: typing.SupportsFloat) -> None:
        ...
    def set_position(self, position: glm.vec3) -> None:
        ...
    def set_x(self, x: typing.SupportsFloat) -> None:
        ...
    def set_y(self, y: typing.SupportsFloat) -> None:
        ...
    def set_yaw(self, yaw: typing.SupportsFloat) -> None:
        ...
    def set_z(self, z: typing.SupportsFloat) -> None:
        ...
    def update(self) -> None:
        ...
    def use(self, shader: ...) -> None:
        ...
class StaticCamera2D:
    def __init__(self, engine: Engine, position: glm.vec2 = (0.0, 0.0), scale: typing.SupportsFloat = 10.0) -> None:
        ...
    def get_position(self) -> glm.vec2:
        ...
    def get_view_height(self) -> float:
        ...
    def get_view_scale(self) -> glm.vec2:
        ...
    def get_view_width(self) -> float:
        ...
    def get_x(self) -> float:
        ...
    def get_y(self) -> float:
        ...
    def set_position(self, position: glm.vec2) -> None:
        ...
    @typing.overload
    def set_scale(self, scale: typing.SupportsFloat) -> None:
        ...
    @typing.overload
    def set_scale(self, x_scale: typing.SupportsFloat, y_scale: typing.SupportsFloat) -> None:
        ...
    @typing.overload
    def set_scale(self, view_scale: glm.vec2) -> None:
        ...
    def set_x(self, x: typing.SupportsFloat) -> None:
        ...
    def set_y(self, y: typing.SupportsFloat) -> None:
        ...
    def update(self) -> None:
        ...
    def use(self, shader: ...) -> None:
        ...
    @property
    def pos(self) -> glm.vec2:
        ...
    @property
    def position(self) -> ...:
        ...
    @position.setter
    def position(self, arg1: typing.Any) -> None:
        ...
    @property
    def scale(self) -> float:
        ...
    @scale.setter
    def scale(self, arg1: typing.SupportsFloat) -> None:
        ...
    @property
    def view_scale(self) -> ...:
        ...
    @view_scale.setter
    def view_scale(self, arg1: typing.Any) -> None:
        ...
class UBO:
    @typing.overload
    def __init__(self, data: typing_extensions.CapsuleType, size: typing.SupportsInt, drawType: typing.SupportsInt) -> None:
        ...
    @typing.overload
    def __init__(self, data: ..., std: ..., drawType: typing.SupportsInt = 35044) -> None:
        ...
    def bind(self) -> None:
        ...
    def get_size(self) -> int:
        ...
    def unbind(self) -> None:
        ...
    def write(self, data: typing_extensions.CapsuleType, size: typing.SupportsInt, offset: typing.SupportsInt) -> None:
        ...
class VAO:
    def __init__(self, shader: Shader, vertices: VBO, indices: EBO) -> None:
        ...
    def bind(self) -> None:
        ...
    def bind_attribute(self, location: typing.SupportsInt, count: typing.SupportsInt, dataType: typing.SupportsInt, stride: typing.SupportsInt, offset: typing.SupportsInt, divisor: typing.SupportsInt) -> None:
        ...
    def bind_attributes(self, attribs: ..., std: ..., std: ..., std: ..., std: ..., std: ..., divisor: typing.SupportsInt) -> None:
        ...
    @typing.overload
    def bind_buffer(self, buffer: VBO, attribs: ..., std: ..., std: ..., std: ..., std: ..., std: ..., divisor: typing.SupportsInt) -> None:
        ...
    @typing.overload
    def bind_buffer(self, buffer: VBO, indices: EBO, attribs: ..., std: ..., std: ..., std: ..., std: ..., std: ..., divisor: typing.SupportsInt) -> None:
        ...
    def render(self, instanceCount: typing.SupportsInt = 0) -> None:
        ...
class VBO:
    @typing.overload
    def __init__(self, data: typing_extensions.CapsuleType, size: typing.SupportsInt, drawType: typing.SupportsInt) -> None:
        ...
    @typing.overload
    def __init__(self, data: collections.abc.Sequence, drawType: typing.SupportsInt = 35044) -> None:
        ...
    def bind(self) -> None:
        ...
    def get_size(self) -> int:
        ...
    def unbind(self) -> None:
        ...
class Window:
    def clear(self, r: typing.SupportsFloat = 0.0, g: typing.SupportsFloat = 0.0, b: typing.SupportsFloat = 0.0, a: typing.SupportsFloat = 1.0) -> None:
        ...
    def disable_blend(self) -> None:
        ...
    def disable_cull_face(self) -> None:
        ...
    def disable_depth_test(self) -> None:
        ...
    def disable_multisample(self) -> None:
        ...
    def disable_v_sync(self) -> None:
        ...
    def enable_blend(self) -> None:
        ...
    def enable_cull_face(self) -> None:
        ...
    def enable_depth_test(self) -> None:
        ...
    def enable_multisample(self) -> None:
        ...
    def enable_v_sync(self) -> None:
        ...
    def get_delta_time(self) -> float:
        ...
    def get_fps(self) -> int:
        ...
    def get_height(self) -> int:
        ...
    def get_time(self) -> float:
        ...
    def get_width(self) -> int:
        ...
    def get_window_scale_x(self) -> float:
        ...
    def get_window_scale_y(self) -> float:
        ...
    def is_running(self) -> bool:
        ...
    def render(self) -> None:
        ...
    def use(self) -> None:
        ...
class _QuatPropertyProxy:
    def __getitem__(self, arg0: typing.SupportsInt) -> float:
        ...
    @typing.overload
    def __imul__(self, arg0: glm.quat) -> _QuatPropertyProxy:
        ...
    @typing.overload
    def __imul__(self, arg0: typing.SupportsFloat) -> _QuatPropertyProxy:
        ...
    def __iter__(self) -> collections.abc.Iterator:
        ...
    def __len__(self) -> int:
        ...
    @typing.overload
    def __mul__(self, arg0: glm.quat) -> glm.quat:
        ...
    @typing.overload
    def __mul__(self, arg0: typing.SupportsFloat) -> glm.quat:
        ...
    def __rmul__(self, arg0: typing.SupportsFloat) -> glm.quat:
        ...
    def __setattr__(self, arg0: str, arg1: typing.Any) -> None:
        ...
    def __setitem__(self, arg0: typing.SupportsInt, arg1: typing.SupportsFloat) -> None:
        ...
    def conjugate(self) -> glm.quat:
        ...
    def dot(self, other: glm.quat) -> float:
        ...
    def euler_angles(self) -> glm.vec3:
        """
        Returns euler angles as a vec3 (pitch, yaw, roll) in radians
        """
    def inverse(self) -> glm.quat:
        ...
    def length(self) -> float:
        ...
    def normalize(self) -> glm.quat:
        ...
    @property
    def w(self) -> float:
        ...
    @w.setter
    def w(self, arg1: typing.SupportsFloat) -> None:
        ...
    @property
    def x(self) -> float:
        ...
    @x.setter
    def x(self, arg1: typing.SupportsFloat) -> None:
        ...
    @property
    def y(self) -> float:
        ...
    @y.setter
    def y(self, arg1: typing.SupportsFloat) -> None:
        ...
    @property
    def z(self) -> float:
        ...
    @z.setter
    def z(self, arg1: typing.SupportsFloat) -> None:
        ...
class _Vec2PropertyProxy:
    @typing.overload
    def __add__(self, arg0: glm.vec2) -> glm.vec2:
        ...
    @typing.overload
    def __add__(self, arg0: typing.Any) -> glm.vec2:
        ...
    def __getitem__(self, arg0: typing.SupportsInt) -> float:
        ...
    @typing.overload
    def __iadd__(self, arg0: glm.vec2) -> _Vec2PropertyProxy:
        ...
    @typing.overload
    def __iadd__(self, arg0: typing.Any) -> _Vec2PropertyProxy:
        ...
    @typing.overload
    def __imul__(self, arg0: typing.SupportsFloat) -> _Vec2PropertyProxy:
        ...
    @typing.overload
    def __imul__(self, arg0: glm.vec2) -> _Vec2PropertyProxy:
        ...
    @typing.overload
    def __imul__(self, arg0: typing.Any) -> _Vec2PropertyProxy:
        ...
    @typing.overload
    def __isub__(self, arg0: glm.vec2) -> _Vec2PropertyProxy:
        ...
    @typing.overload
    def __isub__(self, arg0: typing.Any) -> _Vec2PropertyProxy:
        ...
    def __iter__(self) -> collections.abc.Iterator:
        ...
    @typing.overload
    def __itruediv__(self, arg0: typing.SupportsFloat) -> _Vec2PropertyProxy:
        ...
    @typing.overload
    def __itruediv__(self, arg0: glm.vec2) -> _Vec2PropertyProxy:
        ...
    @typing.overload
    def __itruediv__(self, arg0: typing.Any) -> _Vec2PropertyProxy:
        ...
    def __len__(self) -> int:
        ...
    @typing.overload
    def __mul__(self, arg0: typing.SupportsFloat) -> glm.vec2:
        ...
    @typing.overload
    def __mul__(self, arg0: glm.vec2) -> glm.vec2:
        ...
    @typing.overload
    def __mul__(self, arg0: typing.Any) -> glm.vec2:
        ...
    @typing.overload
    def __radd__(self, arg0: glm.vec2) -> glm.vec2:
        ...
    @typing.overload
    def __radd__(self, arg0: typing.Any) -> glm.vec2:
        ...
    @typing.overload
    def __rmul__(self, arg0: typing.SupportsFloat) -> glm.vec2:
        ...
    @typing.overload
    def __rmul__(self, arg0: typing.Any) -> glm.vec2:
        ...
    @typing.overload
    def __rsub__(self, arg0: glm.vec2) -> glm.vec2:
        ...
    @typing.overload
    def __rsub__(self, arg0: typing.Any) -> glm.vec2:
        ...
    def __setattr__(self, arg0: str, arg1: typing.Any) -> None:
        ...
    def __setitem__(self, arg0: typing.SupportsInt, arg1: typing.SupportsFloat) -> None:
        ...
    @typing.overload
    def __sub__(self, arg0: glm.vec2) -> glm.vec2:
        ...
    @typing.overload
    def __sub__(self, arg0: typing.Any) -> glm.vec2:
        ...
    @typing.overload
    def __truediv__(self, arg0: typing.SupportsFloat) -> glm.vec2:
        ...
    @typing.overload
    def __truediv__(self, arg0: glm.vec2) -> glm.vec2:
        ...
    @typing.overload
    def __truediv__(self, arg0: typing.Any) -> glm.vec2:
        ...
    @property
    def x(self) -> float:
        ...
    @x.setter
    def x(self, arg1: typing.SupportsFloat) -> None:
        ...
    @property
    def y(self) -> float:
        ...
    @y.setter
    def y(self, arg1: typing.SupportsFloat) -> None:
        ...
class _Vec3PropertyProxy:
    @typing.overload
    def __add__(self, arg0: glm.vec3) -> glm.vec3:
        ...
    @typing.overload
    def __add__(self, arg0: typing.Any) -> glm.vec3:
        ...
    def __getitem__(self, arg0: typing.SupportsInt) -> float:
        ...
    @typing.overload
    def __iadd__(self, arg0: glm.vec3) -> _Vec3PropertyProxy:
        ...
    @typing.overload
    def __iadd__(self, arg0: typing.Any) -> _Vec3PropertyProxy:
        ...
    @typing.overload
    def __imul__(self, arg0: typing.SupportsFloat) -> _Vec3PropertyProxy:
        ...
    @typing.overload
    def __imul__(self, arg0: glm.vec3) -> _Vec3PropertyProxy:
        ...
    @typing.overload
    def __imul__(self, arg0: typing.Any) -> _Vec3PropertyProxy:
        ...
    @typing.overload
    def __isub__(self, arg0: glm.vec3) -> _Vec3PropertyProxy:
        ...
    @typing.overload
    def __isub__(self, arg0: typing.Any) -> _Vec3PropertyProxy:
        ...
    def __iter__(self) -> collections.abc.Iterator:
        ...
    @typing.overload
    def __itruediv__(self, arg0: typing.SupportsFloat) -> _Vec3PropertyProxy:
        ...
    @typing.overload
    def __itruediv__(self, arg0: glm.vec3) -> _Vec3PropertyProxy:
        ...
    @typing.overload
    def __itruediv__(self, arg0: typing.Any) -> _Vec3PropertyProxy:
        ...
    def __len__(self) -> int:
        ...
    @typing.overload
    def __mul__(self, arg0: typing.SupportsFloat) -> glm.vec3:
        ...
    @typing.overload
    def __mul__(self, arg0: glm.vec3) -> glm.vec3:
        ...
    @typing.overload
    def __mul__(self, arg0: typing.Any) -> glm.vec3:
        ...
    @typing.overload
    def __radd__(self, arg0: glm.vec3) -> glm.vec3:
        ...
    @typing.overload
    def __radd__(self, arg0: typing.Any) -> glm.vec3:
        ...
    @typing.overload
    def __rmul__(self, arg0: typing.SupportsFloat) -> glm.vec3:
        ...
    @typing.overload
    def __rmul__(self, arg0: typing.Any) -> glm.vec3:
        ...
    @typing.overload
    def __rsub__(self, arg0: glm.vec3) -> glm.vec3:
        ...
    @typing.overload
    def __rsub__(self, arg0: typing.Any) -> glm.vec3:
        ...
    def __setattr__(self, arg0: str, arg1: typing.Any) -> None:
        ...
    def __setitem__(self, arg0: typing.SupportsInt, arg1: typing.SupportsFloat) -> None:
        ...
    @typing.overload
    def __sub__(self, arg0: glm.vec3) -> glm.vec3:
        ...
    @typing.overload
    def __sub__(self, arg0: typing.Any) -> glm.vec3:
        ...
    @typing.overload
    def __truediv__(self, arg0: typing.SupportsFloat) -> glm.vec3:
        ...
    @typing.overload
    def __truediv__(self, arg0: glm.vec3) -> glm.vec3:
        ...
    @typing.overload
    def __truediv__(self, arg0: typing.Any) -> glm.vec3:
        ...
    @property
    def x(self) -> float:
        ...
    @x.setter
    def x(self, arg1: typing.SupportsFloat) -> None:
        ...
    @property
    def y(self) -> float:
        ...
    @y.setter
    def y(self, arg1: typing.SupportsFloat) -> None:
        ...
    @property
    def z(self) -> float:
        ...
    @z.setter
    def z(self, arg1: typing.SupportsFloat) -> None:
        ...
def init_gpu() -> None:
    """
    Initialize the GPU compute backend (call once at startup).
    """
F32: GpuBufferDtype  # value = <GpuBufferDtype.F32: 1>
I32: GpuBufferDtype  # value = <GpuBufferDtype.I32: 2>
U32: GpuBufferDtype  # value = <GpuBufferDtype.U32: 0>



"""
Keyboard key constants
"""
from __future__ import annotations
import typing
__all__: list[str] = ['K_0', 'K_1', 'K_2', 'K_3', 'K_4', 'K_5', 'K_6', 'K_7', 'K_8', 'K_9', 'K_A', 'K_B', 'K_BACKSPACE', 'K_C', 'K_CAPS_LOCK', 'K_D', 'K_E', 'K_ENTER', 'K_ESCAPE', 'K_F', 'K_F1', 'K_F10', 'K_F11', 'K_F12', 'K_F2', 'K_F3', 'K_F4', 'K_F5', 'K_F6', 'K_F7', 'K_F8', 'K_F9', 'K_G', 'K_H', 'K_I', 'K_J', 'K_K', 'K_L', 'K_LEFT_ALT', 'K_LEFT_CONTROL', 'K_LEFT_SHIFT', 'K_M', 'K_N', 'K_O', 'K_P', 'K_Q', 'K_R', 'K_RIGHT_ALT', 'K_RIGHT_CONTROL', 'K_RIGHT_SHIFT', 'K_S', 'K_SPACE', 'K_T', 'K_TAB', 'K_U', 'K_V', 'K_W', 'K_X', 'K_Y', 'K_Z', 'KeyCode']
class KeyCode:
    """
    Members:
    
      K_A
    
      K_B
    
      K_C
    
      K_D
    
      K_E
    
      K_F
    
      K_G
    
      K_H
    
      K_I
    
      K_J
    
      K_K
    
      K_L
    
      K_M
    
      K_N
    
      K_O
    
      K_P
    
      K_Q
    
      K_R
    
      K_S
    
      K_T
    
      K_U
    
      K_V
    
      K_W
    
      K_X
    
      K_Y
    
      K_Z
    
      K_0
    
      K_1
    
      K_2
    
      K_3
    
      K_4
    
      K_5
    
      K_6
    
      K_7
    
      K_8
    
      K_9
    
      K_F1
    
      K_F2
    
      K_F3
    
      K_F4
    
      K_F5
    
      K_F6
    
      K_F7
    
      K_F8
    
      K_F9
    
      K_F10
    
      K_F11
    
      K_F12
    
      K_BACKSPACE
    
      K_TAB
    
      K_ENTER
    
      K_LEFT_SHIFT
    
      K_RIGHT_SHIFT
    
      K_LEFT_CONTROL
    
      K_RIGHT_CONTROL
    
      K_LEFT_ALT
    
      K_RIGHT_ALT
    
      K_CAPS_LOCK
    
      K_ESCAPE
    
      K_SPACE
    """
    K_0: typing.ClassVar[KeyCode]  # value = <KeyCode.K_0: 48>
    K_1: typing.ClassVar[KeyCode]  # value = <KeyCode.K_1: 49>
    K_2: typing.ClassVar[KeyCode]  # value = <KeyCode.K_2: 50>
    K_3: typing.ClassVar[KeyCode]  # value = <KeyCode.K_3: 51>
    K_4: typing.ClassVar[KeyCode]  # value = <KeyCode.K_4: 52>
    K_5: typing.ClassVar[KeyCode]  # value = <KeyCode.K_5: 53>
    K_6: typing.ClassVar[KeyCode]  # value = <KeyCode.K_6: 54>
    K_7: typing.ClassVar[KeyCode]  # value = <KeyCode.K_7: 55>
    K_8: typing.ClassVar[KeyCode]  # value = <KeyCode.K_8: 56>
    K_9: typing.ClassVar[KeyCode]  # value = <KeyCode.K_9: 57>
    K_A: typing.ClassVar[KeyCode]  # value = <KeyCode.K_A: 65>
    K_B: typing.ClassVar[KeyCode]  # value = <KeyCode.K_B: 66>
    K_BACKSPACE: typing.ClassVar[KeyCode]  # value = <KeyCode.K_BACKSPACE: 259>
    K_C: typing.ClassVar[KeyCode]  # value = <KeyCode.K_C: 67>
    K_CAPS_LOCK: typing.ClassVar[KeyCode]  # value = <KeyCode.K_CAPS_LOCK: 280>
    K_D: typing.ClassVar[KeyCode]  # value = <KeyCode.K_D: 68>
    K_E: typing.ClassVar[KeyCode]  # value = <KeyCode.K_E: 69>
    K_ENTER: typing.ClassVar[KeyCode]  # value = <KeyCode.K_ENTER: 257>
    K_ESCAPE: typing.ClassVar[KeyCode]  # value = <KeyCode.K_ESCAPE: 256>
    K_F: typing.ClassVar[KeyCode]  # value = <KeyCode.K_F: 70>
    K_F1: typing.ClassVar[KeyCode]  # value = <KeyCode.K_F1: 290>
    K_F10: typing.ClassVar[KeyCode]  # value = <KeyCode.K_F10: 299>
    K_F11: typing.ClassVar[KeyCode]  # value = <KeyCode.K_F11: 300>
    K_F12: typing.ClassVar[KeyCode]  # value = <KeyCode.K_F12: 301>
    K_F2: typing.ClassVar[KeyCode]  # value = <KeyCode.K_F2: 291>
    K_F3: typing.ClassVar[KeyCode]  # value = <KeyCode.K_F3: 292>
    K_F4: typing.ClassVar[KeyCode]  # value = <KeyCode.K_F4: 293>
    K_F5: typing.ClassVar[KeyCode]  # value = <KeyCode.K_F5: 294>
    K_F6: typing.ClassVar[KeyCode]  # value = <KeyCode.K_F6: 295>
    K_F7: typing.ClassVar[KeyCode]  # value = <KeyCode.K_F7: 296>
    K_F8: typing.ClassVar[KeyCode]  # value = <KeyCode.K_F8: 297>
    K_F9: typing.ClassVar[KeyCode]  # value = <KeyCode.K_F9: 298>
    K_G: typing.ClassVar[KeyCode]  # value = <KeyCode.K_G: 71>
    K_H: typing.ClassVar[KeyCode]  # value = <KeyCode.K_H: 72>
    K_I: typing.ClassVar[KeyCode]  # value = <KeyCode.K_I: 73>
    K_J: typing.ClassVar[KeyCode]  # value = <KeyCode.K_J: 74>
    K_K: typing.ClassVar[KeyCode]  # value = <KeyCode.K_K: 75>
    K_L: typing.ClassVar[KeyCode]  # value = <KeyCode.K_L: 76>
    K_LEFT_ALT: typing.ClassVar[KeyCode]  # value = <KeyCode.K_LEFT_ALT: 342>
    K_LEFT_CONTROL: typing.ClassVar[KeyCode]  # value = <KeyCode.K_LEFT_CONTROL: 341>
    K_LEFT_SHIFT: typing.ClassVar[KeyCode]  # value = <KeyCode.K_LEFT_SHIFT: 340>
    K_M: typing.ClassVar[KeyCode]  # value = <KeyCode.K_M: 77>
    K_N: typing.ClassVar[KeyCode]  # value = <KeyCode.K_N: 78>
    K_O: typing.ClassVar[KeyCode]  # value = <KeyCode.K_O: 79>
    K_P: typing.ClassVar[KeyCode]  # value = <KeyCode.K_P: 80>
    K_Q: typing.ClassVar[KeyCode]  # value = <KeyCode.K_Q: 81>
    K_R: typing.ClassVar[KeyCode]  # value = <KeyCode.K_R: 82>
    K_RIGHT_ALT: typing.ClassVar[KeyCode]  # value = <KeyCode.K_RIGHT_ALT: 346>
    K_RIGHT_CONTROL: typing.ClassVar[KeyCode]  # value = <KeyCode.K_RIGHT_CONTROL: 345>
    K_RIGHT_SHIFT: typing.ClassVar[KeyCode]  # value = <KeyCode.K_RIGHT_SHIFT: 344>
    K_S: typing.ClassVar[KeyCode]  # value = <KeyCode.K_S: 83>
    K_SPACE: typing.ClassVar[KeyCode]  # value = <KeyCode.K_SPACE: 32>
    K_T: typing.ClassVar[KeyCode]  # value = <KeyCode.K_T: 84>
    K_TAB: typing.ClassVar[KeyCode]  # value = <KeyCode.K_TAB: 258>
    K_U: typing.ClassVar[KeyCode]  # value = <KeyCode.K_U: 85>
    K_V: typing.ClassVar[KeyCode]  # value = <KeyCode.K_V: 86>
    K_W: typing.ClassVar[KeyCode]  # value = <KeyCode.K_W: 87>
    K_X: typing.ClassVar[KeyCode]  # value = <KeyCode.K_X: 88>
    K_Y: typing.ClassVar[KeyCode]  # value = <KeyCode.K_Y: 89>
    K_Z: typing.ClassVar[KeyCode]  # value = <KeyCode.K_Z: 90>
    __members__: typing.ClassVar[dict[str, KeyCode]]  # value = {'K_A': <KeyCode.K_A: 65>, 'K_B': <KeyCode.K_B: 66>, 'K_C': <KeyCode.K_C: 67>, 'K_D': <KeyCode.K_D: 68>, 'K_E': <KeyCode.K_E: 69>, 'K_F': <KeyCode.K_F: 70>, 'K_G': <KeyCode.K_G: 71>, 'K_H': <KeyCode.K_H: 72>, 'K_I': <KeyCode.K_I: 73>, 'K_J': <KeyCode.K_J: 74>, 'K_K': <KeyCode.K_K: 75>, 'K_L': <KeyCode.K_L: 76>, 'K_M': <KeyCode.K_M: 77>, 'K_N': <KeyCode.K_N: 78>, 'K_O': <KeyCode.K_O: 79>, 'K_P': <KeyCode.K_P: 80>, 'K_Q': <KeyCode.K_Q: 81>, 'K_R': <KeyCode.K_R: 82>, 'K_S': <KeyCode.K_S: 83>, 'K_T': <KeyCode.K_T: 84>, 'K_U': <KeyCode.K_U: 85>, 'K_V': <KeyCode.K_V: 86>, 'K_W': <KeyCode.K_W: 87>, 'K_X': <KeyCode.K_X: 88>, 'K_Y': <KeyCode.K_Y: 89>, 'K_Z': <KeyCode.K_Z: 90>, 'K_0': <KeyCode.K_0: 48>, 'K_1': <KeyCode.K_1: 49>, 'K_2': <KeyCode.K_2: 50>, 'K_3': <KeyCode.K_3: 51>, 'K_4': <KeyCode.K_4: 52>, 'K_5': <KeyCode.K_5: 53>, 'K_6': <KeyCode.K_6: 54>, 'K_7': <KeyCode.K_7: 55>, 'K_8': <KeyCode.K_8: 56>, 'K_9': <KeyCode.K_9: 57>, 'K_F1': <KeyCode.K_F1: 290>, 'K_F2': <KeyCode.K_F2: 291>, 'K_F3': <KeyCode.K_F3: 292>, 'K_F4': <KeyCode.K_F4: 293>, 'K_F5': <KeyCode.K_F5: 294>, 'K_F6': <KeyCode.K_F6: 295>, 'K_F7': <KeyCode.K_F7: 296>, 'K_F8': <KeyCode.K_F8: 297>, 'K_F9': <KeyCode.K_F9: 298>, 'K_F10': <KeyCode.K_F10: 299>, 'K_F11': <KeyCode.K_F11: 300>, 'K_F12': <KeyCode.K_F12: 301>, 'K_BACKSPACE': <KeyCode.K_BACKSPACE: 259>, 'K_TAB': <KeyCode.K_TAB: 258>, 'K_ENTER': <KeyCode.K_ENTER: 257>, 'K_LEFT_SHIFT': <KeyCode.K_LEFT_SHIFT: 340>, 'K_RIGHT_SHIFT': <KeyCode.K_RIGHT_SHIFT: 344>, 'K_LEFT_CONTROL': <KeyCode.K_LEFT_CONTROL: 341>, 'K_RIGHT_CONTROL': <KeyCode.K_RIGHT_CONTROL: 345>, 'K_LEFT_ALT': <KeyCode.K_LEFT_ALT: 342>, 'K_RIGHT_ALT': <KeyCode.K_RIGHT_ALT: 346>, 'K_CAPS_LOCK': <KeyCode.K_CAPS_LOCK: 280>, 'K_ESCAPE': <KeyCode.K_ESCAPE: 256>, 'K_SPACE': <KeyCode.K_SPACE: 32>}
    def __eq__(self, other: typing.Any) -> bool:
        ...
    def __getstate__(self) -> int:
        ...
    def __hash__(self) -> int:
        ...
    def __index__(self) -> int:
        ...
    def __init__(self, value: typing.SupportsInt) -> None:
        ...
    def __int__(self) -> int:
        ...
    def __ne__(self, other: typing.Any) -> bool:
        ...
    def __repr__(self) -> str:
        ...
    def __setstate__(self, state: typing.SupportsInt) -> None:
        ...
    def __str__(self) -> str:
        ...
    @property
    def name(self) -> str:
        ...
    @property
    def value(self) -> int:
        ...
K_0: KeyCode  # value = <KeyCode.K_0: 48>
K_1: KeyCode  # value = <KeyCode.K_1: 49>
K_2: KeyCode  # value = <KeyCode.K_2: 50>
K_3: KeyCode  # value = <KeyCode.K_3: 51>
K_4: KeyCode  # value = <KeyCode.K_4: 52>
K_5: KeyCode  # value = <KeyCode.K_5: 53>
K_6: KeyCode  # value = <KeyCode.K_6: 54>
K_7: KeyCode  # value = <KeyCode.K_7: 55>
K_8: KeyCode  # value = <KeyCode.K_8: 56>
K_9: KeyCode  # value = <KeyCode.K_9: 57>
K_A: KeyCode  # value = <KeyCode.K_A: 65>
K_B: KeyCode  # value = <KeyCode.K_B: 66>
K_BACKSPACE: KeyCode  # value = <KeyCode.K_BACKSPACE: 259>
K_C: KeyCode  # value = <KeyCode.K_C: 67>
K_CAPS_LOCK: KeyCode  # value = <KeyCode.K_CAPS_LOCK: 280>
K_D: KeyCode  # value = <KeyCode.K_D: 68>
K_E: KeyCode  # value = <KeyCode.K_E: 69>
K_ENTER: KeyCode  # value = <KeyCode.K_ENTER: 257>
K_ESCAPE: KeyCode  # value = <KeyCode.K_ESCAPE: 256>
K_F: KeyCode  # value = <KeyCode.K_F: 70>
K_F1: KeyCode  # value = <KeyCode.K_F1: 290>
K_F10: KeyCode  # value = <KeyCode.K_F10: 299>
K_F11: KeyCode  # value = <KeyCode.K_F11: 300>
K_F12: KeyCode  # value = <KeyCode.K_F12: 301>
K_F2: KeyCode  # value = <KeyCode.K_F2: 291>
K_F3: KeyCode  # value = <KeyCode.K_F3: 292>
K_F4: KeyCode  # value = <KeyCode.K_F4: 293>
K_F5: KeyCode  # value = <KeyCode.K_F5: 294>
K_F6: KeyCode  # value = <KeyCode.K_F6: 295>
K_F7: KeyCode  # value = <KeyCode.K_F7: 296>
K_F8: KeyCode  # value = <KeyCode.K_F8: 297>
K_F9: KeyCode  # value = <KeyCode.K_F9: 298>
K_G: KeyCode  # value = <KeyCode.K_G: 71>
K_H: KeyCode  # value = <KeyCode.K_H: 72>
K_I: KeyCode  # value = <KeyCode.K_I: 73>
K_J: KeyCode  # value = <KeyCode.K_J: 74>
K_K: KeyCode  # value = <KeyCode.K_K: 75>
K_L: KeyCode  # value = <KeyCode.K_L: 76>
K_LEFT_ALT: KeyCode  # value = <KeyCode.K_LEFT_ALT: 342>
K_LEFT_CONTROL: KeyCode  # value = <KeyCode.K_LEFT_CONTROL: 341>
K_LEFT_SHIFT: KeyCode  # value = <KeyCode.K_LEFT_SHIFT: 340>
K_M: KeyCode  # value = <KeyCode.K_M: 77>
K_N: KeyCode  # value = <KeyCode.K_N: 78>
K_O: KeyCode  # value = <KeyCode.K_O: 79>
K_P: KeyCode  # value = <KeyCode.K_P: 80>
K_Q: KeyCode  # value = <KeyCode.K_Q: 81>
K_R: KeyCode  # value = <KeyCode.K_R: 82>
K_RIGHT_ALT: KeyCode  # value = <KeyCode.K_RIGHT_ALT: 346>
K_RIGHT_CONTROL: KeyCode  # value = <KeyCode.K_RIGHT_CONTROL: 345>
K_RIGHT_SHIFT: KeyCode  # value = <KeyCode.K_RIGHT_SHIFT: 344>
K_S: KeyCode  # value = <KeyCode.K_S: 83>
K_SPACE: KeyCode  # value = <KeyCode.K_SPACE: 32>
K_T: KeyCode  # value = <KeyCode.K_T: 84>
K_TAB: KeyCode  # value = <KeyCode.K_TAB: 258>
K_U: KeyCode  # value = <KeyCode.K_U: 85>
K_V: KeyCode  # value = <KeyCode.K_V: 86>
K_W: KeyCode  # value = <KeyCode.K_W: 87>
K_X: KeyCode  # value = <KeyCode.K_X: 88>
K_Y: KeyCode  # value = <KeyCode.K_Y: 89>
K_Z: KeyCode  # value = <KeyCode.K_Z: 90>



"""
Using a post-processing shader
"""

import basilisk as bsk

engine = bsk.Engine()
scene = bsk.Scene(engine)

mesh = bsk.Mesh("models/sphere.obj")
node = bsk.Node(scene, mesh)

shader = bsk.Shader("shaders/frame.vert", "shaders/post.frag")
frame = bsk.Frame(engine, shader, 160, 160)
frame.set_filter_nearest()

while engine.is_running():
    engine.update()
    scene.update()

    frame.clear()
    frame.use()
    scene.render()

    engine.get_frame().use()
    frame.render()
    engine.render()


"""
Using a custom shader
"""

import basilisk as bsk

engine = bsk.Engine()

shader = bsk.Shader("shaders/test.vert", "shaders/test.frag")
scene = bsk.Scene(engine, shader)

mesh = bsk.Mesh("models/sphere.obj")
node = bsk.Node(scene, mesh)

while engine.is_running():
    engine.update()
    scene.update()
    scene.render()
    engine.render()


#!/usr/bin/env python3
"""
Build script for PyInstaller that:
1. Runs PyInstaller on a given file or spec
2. Copies all top-level files and folders (excluding PyInstaller-generated ones)
   into the dist/.../_internal/ folder

Can be run directly or via: basilisk file.py / basilisk-engine file.py
"""

import sys
from pathlib import Path

# Allow running from repo without installing (add parent to path for basilisk package)
if __name__ == '__main__':
    _root = Path(__file__).resolve().parent.parent
    if str(_root) not in sys.path:
        sys.path.insert(0, str(_root))

from basilisk.build import main

if __name__ == '__main__':
    main()



#!/usr/bin/env python3
"""
Build script for cx_Freeze that:
1. Runs cx_Freeze on a given Python script
2. Copies all top-level files and folders (excluding build-generated ones) 
   into the output folder
3. Reorganizes output to a clean 'build/' directory structure
"""

import argparse
import os
import platform
import shutil
import subprocess
import sys
from pathlib import Path


def get_cxfreeze_build_dir():
    """
    Determine the cx_Freeze build directory name based on platform.
    cx_Freeze creates directories like: exe.linux-x86_64-3.10
    """
    system = platform.system().lower()
    machine = platform.machine()
    py_version = f"{sys.version_info.major}.{sys.version_info.minor}"
    
    return f"exe.{system}-{machine}-{py_version}"


def create_setup_script(script_path, work_dir, output_name):
    """
    Create a temporary setup.py for cx_Freeze.
    Returns the path to the created setup.py.
    """
    setup_content = f'''"""
Auto-generated cx_Freeze setup script
"""

import sys
from cx_Freeze import setup, Executable

build_exe_options = {{
    "packages": ["glm", "glcontext"],
    "includes": [],
    "excludes": ["tkinter", "unittest", "email", "html", "http", "xml", "pydoc"],
}}

base = None

executables = [
    Executable(
        "{script_path.name}",
        base=base,
        target_name="{output_name}",
    )
]

setup(
    name="{output_name}",
    version="1.0",
    description="Built with cx_Freeze",
    options={{"build_exe": build_exe_options}},
    executables=executables,
)
'''
    
    setup_path = work_dir / "_temp_setup.py"
    with open(setup_path, 'w') as f:
        f.write(setup_content)
    
    return setup_path


def run_cxfreeze(script_path, work_dir=None):
    """Run cx_Freeze on the given Python script."""
    if work_dir is None:
        work_dir = Path(script_path).parent
    
    work_dir = Path(work_dir)
    script_path = Path(script_path)
    
    # Make path absolute
    if not script_path.is_absolute():
        script_path = work_dir / script_path
    
    output_name = script_path.stem
    
    print(f"Running cx_Freeze on: {script_path}")
    print(f"Working directory: {work_dir}")
    
    # Create temporary setup.py
    setup_path = create_setup_script(script_path, work_dir, output_name)
    
    try:
        # Run cx_Freeze build
        result = subprocess.run(
            [sys.executable, str(setup_path), "build"],
            cwd=work_dir,
            check=True
        )
    finally:
        # Clean up temporary setup.py
        if setup_path.exists():
            setup_path.unlink()
    
    return result


def copy_top_level_files(source_dir, dest_dir, exclude_patterns=None):
    """
    Copy all top-level files and folders from source_dir to dest_dir,
    excluding build-generated files and patterns in exclude_patterns.
    """
    if exclude_patterns is None:
        exclude_patterns = []
    
    # Default exclusions for build-generated files
    default_excludes = {
        'build',
        'dist',
        '__pycache__',
        '*.pyc',
        '*.pyo',
        '*.spec',
        '*.py',
        '*.txt',
        '*.md',
        '*.github',
        '*.git',
        '*.gitkeep',
        '.gitignore',
        '.pytest_cache',
        '.mypy_cache',
        '*.egg-info',
        '_temp_setup.py',
        'build_cxfreeze.py',
        'build_pyinstaller.py',
    }
    
    # Combine with user-provided exclusions
    all_excludes = set(default_excludes) | set(exclude_patterns)
    
    source_dir = Path(source_dir)
    dest_dir = Path(dest_dir)
    
    if not source_dir.exists():
        print(f"Warning: Source directory {source_dir} does not exist")
        return
    
    dest_dir.mkdir(parents=True, exist_ok=True)
    
    print(f"Copying files from {source_dir} to {dest_dir}")
    
    copied_count = 0
    skipped_count = 0
    
    for item in source_dir.iterdir():
        # Skip if matches exclusion pattern
        should_exclude = False
        for pattern in all_excludes:
            if pattern.startswith('*.'):
                # File extension pattern
                if item.suffix == pattern[1:] or item.name.endswith(pattern[1:]):
                    should_exclude = True
                    break
            elif pattern.endswith('*'):
                # Wildcard pattern
                if item.name.startswith(pattern[:-1]):
                    should_exclude = True
                    break
            elif item.name == pattern:
                should_exclude = True
                break
        
        if should_exclude:
            print(f"  Skipping: {item.name}")
            skipped_count += 1
            continue
        
        dest_item = dest_dir / item.name
        
        try:
            if item.is_file():
                shutil.copy2(item, dest_item)
                print(f"  Copied file: {item.name}")
                copied_count += 1
            elif item.is_dir():
                if dest_item.exists():
                    shutil.rmtree(dest_item)
                shutil.copytree(item, dest_item, dirs_exist_ok=True)
                print(f"  Copied directory: {item.name}/")
                copied_count += 1
        except Exception as e:
            print(f"  Error copying {item.name}: {e}")
    
    print(f"\nCopy complete: {copied_count} items copied, {skipped_count} items skipped")


def copy_basilisk_resources(dest_dir):
    """
    Copy basilisk package resources (shaders, models, textures) to dest_dir/lib/.
    These are needed at runtime for the engine to find default resources.
    """
    try:
        import basilisk
        basilisk_package_dir = Path(basilisk.__file__).parent
    except ImportError:
        print("Warning: Could not import basilisk package. Skipping resource copy.")
        print("  Make sure basilisk is installed: pip install -e .")
        return
    
    # The resources should be in the same directory as the .so/.pyd file
    # (shaders/, models/, textures/ directories)
    basilisk_resources = ['shaders', 'models', 'textures']
    basilisk_dest = dest_dir / 'lib'
    basilisk_dest.mkdir(parents=True, exist_ok=True)
    
    print(f"Copying basilisk resources from {basilisk_package_dir} to {basilisk_dest}")
    
    copied_count = 0
    for resource_dir in basilisk_resources:
        source_path = basilisk_package_dir / resource_dir
        dest_path = basilisk_dest / resource_dir
        
        if source_path.exists() and source_path.is_dir():
            if dest_path.exists():
                shutil.rmtree(dest_path)
            shutil.copytree(source_path, dest_path, dirs_exist_ok=True)
            print(f"  Copied: {resource_dir}/")
            copied_count += 1
        else:
            print(f"  Warning: {resource_dir}/ not found in {basilisk_package_dir}")
    
    if copied_count == 0:
        print("  Warning: No basilisk resources found. The application may not work correctly.")
    else:
        print(f"  Successfully copied {copied_count} resource directories")


def main():
    parser = argparse.ArgumentParser(
        description='Build cx_Freeze bundle and copy top-level files'
    )
    parser.add_argument(
        'file',
        nargs='?',
        default='main.py',
        help='Python script to build (default: main.py)'
    )
    parser.add_argument(
        '--work-dir',
        type=str,
        default=None,
        help='Working directory (default: directory containing the file)'
    )
    parser.add_argument(
        '--source-dir',
        type=str,
        default=None,
        help='Source directory to copy files from (default: work-dir)'
    )
    parser.add_argument(
        '--exclude',
        action='append',
        default=[],
        help='Additional patterns to exclude (can be used multiple times)'
    )
    parser.add_argument(
        '--no-copy',
        action='store_true',
        help='Skip copying files, only run cx_Freeze'
    )
    
    args = parser.parse_args()
    
    # Determine working directory
    file_path = Path(args.file)
    if not file_path.is_absolute():
        file_path = Path.cwd() / file_path
    
    if args.work_dir:
        work_dir = Path(args.work_dir)
    else:
        work_dir = file_path.parent
    
    if args.source_dir:
        source_dir = Path(args.source_dir)
    else:
        source_dir = work_dir
    
    output_name = file_path.stem
    
    # Step 1: Run cx_Freeze
    print("=" * 60)
    print("Step 1: Running cx_Freeze")
    print("=" * 60)
    try:
        run_cxfreeze(file_path, work_dir)
    except subprocess.CalledProcessError as e:
        print(f"Error: cx_Freeze failed with exit code {e.returncode}")
        sys.exit(1)
    
    # Step 2: Find the cx_Freeze output directory
    # cx_Freeze creates: build/exe.<platform>-<version>/
    cxfreeze_build_subdir = get_cxfreeze_build_dir()
    cxfreeze_output_dir = work_dir / 'build' / cxfreeze_build_subdir
    
    if not cxfreeze_output_dir.exists():
        # Try to find it by pattern matching
        build_dir = work_dir / 'build'
        if build_dir.exists():
            for item in build_dir.iterdir():
                if item.is_dir() and item.name.startswith('exe.'):
                    cxfreeze_output_dir = item
                    break
    
    if not cxfreeze_output_dir.exists():
        print(f"\nError: cx_Freeze output directory not found")
        print(f"Expected: {cxfreeze_output_dir}")
        print("Please check the build/ directory manually.")
        sys.exit(1)
    
    print(f"\nFound cx_Freeze output: {cxfreeze_output_dir}")
    
    # Step 2: Copy basilisk package resources
    print("\n" + "=" * 60)
    print("Step 2: Copying basilisk package resources")
    print("=" * 60)
    copy_basilisk_resources(cxfreeze_output_dir)
    
    # Step 3: Copy files to output directory (if not skipped)
    if not args.no_copy:
        print("\n" + "=" * 60)
        print("Step 3: Copying top-level files to build output")
        print("=" * 60)
        copy_top_level_files(source_dir, cxfreeze_output_dir, args.exclude)
    
    # Step 4: Reorganize to final structure
    print("\n" + "=" * 60)
    print("Step 4: Reorganizing output structure")
    print("=" * 60)
    
    # Target location: top-level 'build' folder with just the contents
    final_build_dir = work_dir / 'build'
    temp_output_dir = work_dir / f'_temp_build_{output_name}'
    
    # Move cx_Freeze output to temp location
    print(f"Moving {cxfreeze_output_dir} to temporary location")
    shutil.move(str(cxfreeze_output_dir), str(temp_output_dir))
    
    # Remove the build directory (which now only has the exe.* folder gone)
    if final_build_dir.exists():
        # Check if there are other items in build/
        remaining = list(final_build_dir.iterdir())
        if not remaining:
            final_build_dir.rmdir()
            print(f"Removed empty build directory")
        else:
            # Remove remaining cx_Freeze artifacts
            for item in remaining:
                if item.is_dir():
                    shutil.rmtree(item)
                else:
                    item.unlink()
            final_build_dir.rmdir()
            print(f"Cleaned up build directory")
    
    # Move temp to final build location
    print(f"Moving to final location: {final_build_dir}")
    shutil.move(str(temp_output_dir), str(final_build_dir))
    
    # Rename executable if needed (cx_Freeze should already use the right name)
    executable_path = final_build_dir / output_name
    if not executable_path.exists():
        # Try to find the executable
        for item in final_build_dir.iterdir():
            if item.is_file() and os.access(item, os.X_OK) and not item.suffix:
                print(f"Renaming {item.name} to {output_name}")
                item.rename(executable_path)
                break
    
    # Clean up .egg-info directories created by cx_Freeze
    print("Cleaning up build artifacts...")
    for item in work_dir.iterdir():
        if item.is_dir() and item.name.endswith('.egg-info'):
            shutil.rmtree(item)
            print(f"  Removed: {item.name}")
    
    print("\n" + "=" * 60)
    print("Build complete!")
    print(f"Final output directory: {final_build_dir}")
    print(f"Executable: {executable_path}")
    print("=" * 60)


if __name__ == '__main__':
    main()




#!/usr/bin/env python3
"""Wrapper script to run pybind11-stubgen, catching errors to make it non-fatal."""
import os
import sys
import subprocess

# Set up PYTHONPATH
pythonpath = os.environ.get('PYTHONPATH', '')
build_dir = sys.argv[1]
output_dir = sys.argv[2]
os.environ['PYTHONPATH'] = build_dir + os.pathsep + pythonpath if pythonpath else build_dir

# Run stubgen, but don't fail if it errors (e.g., missing runtime dependencies)
try:
    result = subprocess.run(
        [sys.executable, '-m', 'pybind11_stubgen', '--output-dir', output_dir, 'basilisk', '--ignore-all-errors'],
        cwd=build_dir,
        capture_output=False  # Let output go through
    )
    # Always exit successfully so build continues
    sys.exit(0)
except Exception as e:
    # Log error but don't fail build
    print(f"Warning: Stub generation failed (non-fatal): {e}", file=sys.stderr)
    sys.exit(0)

